from __future__ import absolute_import, division, print_function
import argparse
import datetime
# import detector simulation modules
import NuRadioReco.modules.efieldToVoltageConverter
import NuRadioReco.modules.channelResampler
import NuRadioReco.modules.channelGenericNoiseAdder
import NuRadioReco.modules.triggerTimeAdjuster
import NuRadioReco.modules.ARIANNA.hardwareResponseIncorporator
import NuRadioReco.modules.trigger.highLowThreshold
from NuRadioReco.utilities import units
from NuRadioMC.simulation import simulation2 as simulation
import logging
logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger("runstrawman")

# initialize detector sim modules
efieldToVoltageConverter = NuRadioReco.modules.efieldToVoltageConverter.efieldToVoltageConverter()
efieldToVoltageConverter.begin()
channelResampler = NuRadioReco.modules.channelResampler.channelResampler()
channelGenericNoiseAdder = NuRadioReco.modules.channelGenericNoiseAdder.channelGenericNoiseAdder()
hardwareResponseIncorporator = NuRadioReco.modules.ARIANNA.hardwareResponseIncorporator.hardwareResponseIncorporator()
triggerSimulator = NuRadioReco.modules.trigger.highLowThreshold.triggerSimulator()
triggerTimeAdjuster = NuRadioReco.modules.triggerTimeAdjuster.triggerTimeAdjuster()
triggerSimulator.begin(log_level=logging.WARNING)

class mySimulation(simulation.simulation):


    def _detector_simulation(self):
        if(bool(self._cfg['signal']['zerosignal'])):
            self._increase_signal(None, 0)
        
        efieldToVoltageConverter.run(self._evt, self._station, self._det)  # convolve efield with antenna pattern
        # downsample trace to internal simulation sampling rate (the efieldToVoltageConverter upsamples the trace to
        # 20 GHz by default to achive a good time resolution when the two signals from the two signal paths are added) 
        channelResampler.run(self._evt, self._station, self._det, sampling_rate=1. / self._dt)
        
        if bool(self._cfg['noise']):
            Vrms = self._Vrms / (self._bandwidth /( 2.5 * units.GHz))** 0.5  # normalize noise level to the bandwidth its generated for
            channelGenericNoiseAdder.run(self._evt, self._station, self._det, amplitude=Vrms, min_freq=0 * units.MHz,
                                         max_freq=2.5 * units.GHz, type='rayleigh')
        

        hardwareResponseIncorporator.run(self._evt, self._station, self._det, sim_to_data=True)
        
        triggerSimulator.run(self._evt, self._station, self._det,
                           threshold_high=0.5 * units.mV,
                           threshold_low=-0.5 * units.mV,
                           high_low_window=5 * units.ns,
                           coinc_window=30 * units.ns,
                           number_concidences=2,
                           triggered_channels=range(8))
        triggerTimeAdjuster.run(self._evt, self._station, self._det)
        
#         self._station.set_triggered(True)
                
        # downsample trace back to detector sampling rate
        channelResampler.run(self._evt, self._station, self._det, sampling_rate=self._sampling_rate_detector)


parser = argparse.ArgumentParser(description='Run NuRadioMC simulation')
parser.add_argument('inputfilename', type=str,
                    help='path to NuRadioMC input event list')
parser.add_argument('detectordescription', type=str,
                    help='path to file containing the detector description')
parser.add_argument('config', type=str,
                    help='NuRadioMC yaml config file')
parser.add_argument('outputfilename', type=str,
                    help='hdf5 output filename')
parser.add_argument('outputfilenameNuRadioReco', type=str, nargs='?', default=None,
                    help='outputfilename of NuRadioReco detector sim file')
args = parser.parse_args()

sim = mySimulation(eventlist=args.inputfilename,
                            outputfilename=args.outputfilename,
                            detectorfile=args.detectordescription,
                            outputfilenameNuRadioReco=args.outputfilenameNuRadioReco,
                            config_file=args.config,
                            log_level=logging.DEBUG,
                            evt_time=datetime.datetime(2018, 12, 30))
sim.run()




import numpy as np
import scipy.constants
from NuRadioMC.utilities import units
from NuRadioMC.utilities import cross_sections
import logging
logger = logging.getLogger('fluxes')



def get_interaction_length(energy,flavors=12, inttype='total', cross_section_type = 'ctw'):
    """
    return interaction length at given energy, assuming a constant ice density
    """
    ice_density = 0.917 # units.g/units.cm**3 water equivalent is 1, so unitless
    N_A = scipy.constants.Avogadro * units.cm**-3
    L = 1./(N_A * ice_density * cross_sections.get_nu_cross_section(energy, flavors=flavors,inttype=inttype,cross_section_type=cross_section_type))
    return L

def get_limit_from_aeff(energy, aeff,
                        livetime,
                        signalEff = 1.00,
                        energyBinsPerDecade=1.000,
                        upperLimOnEvents=2.44):

    """
    Limit from effective volume

    Parameters:
        --------------
    energy: array of floats
        neutrino energy
    veff: array of floats
        effective volumes
    livetime: float
        time used
    signalEff: float
        efficiency of signal reconstruction
    energyBinsPerDecade: float
        1 for decade bins, 2 for half-decade bins, etc.
    upperLimOnEvents: float
         2.3 for Neyman UL w/ 0 background,
         2.44 for F-C UL w/ 0 background, etc

    """

    evtsPerFluxPerEnergy = aeff * signalEff
    evtsPerFluxPerEnergy *= livetime

    ul  = upperLimOnEvents / evtsPerFluxPerEnergy
    ul *= energyBinsPerDecade / np.log(10)
    ul /= energy

    return ul



def get_limit_flux(energy, veff,
                    livetime,
                    signalEff = 1.00,
                    energyBinsPerDecade=1.000,
                    upperLimOnEvents=2.44,
                    nuCrsScn='ctw'):

    """
    Limit from effective volume

    Parameters:
        --------------
    energy: array of floats
        neutrino energy
    veff: array of floats
        effective volumes
    livetime: float
        time used
    signalEff: float
        efficiency of signal reconstruction
    energyBinsPerDecade: float
        1 for decade bins, 2 for half-decade bins, etc.
    upperLimOnEvents: float
         2.3 for Neyman UL w/ 0 background,
         2.44 for F-C UL w/ 0 background, etc
    nuCrsScn: str
        type of neutrino cross-section


    """

    evtsPerFluxPerEnergy = veff * signalEff
    evtsPerFluxPerEnergy *= livetime
    evtsPerFluxPerEnergy /= get_interaction_length(energy, cross_section_type = nuCrsScn)

    ul  = upperLimOnEvents / evtsPerFluxPerEnergy
    ul *= energyBinsPerDecade / np.log(10)
    ul /= energy

    return ul

# def get_integrated_limit_flux(energy, veff,
#                               livetime,
#                               signalEff = 1.00,
#                               upperLimOnEvents=2.44,
#                               nuCrsScn='ctw'):
#
#     """
#     Limit from effective volume on E^2 flux plot
#
#     Parameters:
#         --------------
#     energy: array of floats
#         energies (the bin centers), the binning in log10(E) must be equidistant!
#     veff: array of floats
#         effective volumes
#     livetime: float
#         time used
#     signalEff: float
#         efficiency of signal reconstruction
#     energyBinsPerDecade: float
#         1 for decade bins, 2 for half-decade bins, etc.
#     upperLimOnEvents: float
#          2.3 for Neyman UL w/ 0 background,
#          2.44 for F-C UL w/ 0 background, etc
#     nuCrsScn: str
#         type of neutrino cross-section
#
#
#     """
#     energy = np.array(energy)
#     veff = np.array(veff)
#     logE = np.log10(energy)
#     dlogE = logE[1] - logE[0]
#
#     L = get_interaction_length(energy, cross_section_type = nuCrsScn)
# #     N = np.sum(veff/L/E) * np.log(10) * livetime * signalEff * dlogE
#     ul = upperLimOnEvents * dlogE/ (livetime * signalEff * np.log(10)) * np.sum(L/veff * energy) / len(energy)
#
#     return ul

def get_limit_e1_flux(energy, veff,
                    livetime,
                    signalEff = 1.00,
                    energyBinsPerDecade=1.000,
                    upperLimOnEvents=2.44,
                    nuCrsScn='ctw'):

    """
    Limit from effective volume on E^1 flux plot

    Parameters:
        --------------
    energy: array of floats
        neutrino energy
    veff: array of floats
        effective volumes
    livetime: float
        time used
    signalEff: float
        efficiency of signal reconstruction
    energyBinsPerDecade: float
        1 for decade bins, 2 for half-decade bins, etc.
    upperLimOnEvents: float
         2.3 for Neyman UL w/ 0 background,
         2.44 for F-C UL w/ 0 background, etc
    nuCrsScn: str
        type of neutrino cross-section


    """

    evtsPerFluxPerEnergy = veff * signalEff
    evtsPerFluxPerEnergy *= livetime
    evtsPerFluxPerEnergy /= get_interaction_length(energy, cross_section_type = nuCrsScn)

    ul  = upperLimOnEvents / evtsPerFluxPerEnergy
    ul *= energyBinsPerDecade / np.log(10)

    return ul

def get_limit_e2_flux(energy, veff,
                    livetime,
                    signalEff = 1.00,
                    energyBinsPerDecade=1.000,
                    upperLimOnEvents=2.44,
                    nuCrsScn='ctw'):
    """
    Limit from effective volume on E^2 flux plot

    Parameters:
        --------------
    energy: array of floats
        neutrino energy
    veff: array of floats
        effective volumes
    livetime: float
        time used
    signalEff: float
        efficiency of signal reconstruction
    energyBinsPerDecade: float
        1 for decade bins, 2 for half-decade bins, etc.
    upperLimOnEvents: float
         2.3 for Neyman UL w/ 0 background,
         2.44 for F-C UL w/ 0 background, etc
    nuCrsScn: str
        type of neutrino cross-section


    """
    return energy**2 * get_limit_flux(energy, veff, livetime, signalEff, energyBinsPerDecade, upperLimOnEvents,nuCrsScn)


def get_number_of_events_for_flux(energies, flux, Veff, livetime, nuCrsScn='ctw'):
    """
    calculates the number of expected neutrinos for a certain flux assumption

    Parameters
    -----------
    energies: array of floats
        energies (the bin centers), the binning in log10(E) must be equidistant!
    flux: array of floats
        the flux at energy logE
    Veff: array of floats
        the effective volume per energy logE
    livetime: float
        the livetime of the detector (including signal efficiency)

    Returns
    -------
    array of floats: number of events per energy bin
    """
    energies = np.array(energies)
    Veff = np.array(Veff)
    logE = np.log10(energies)
    dlogE = logE[1] - logE[0]
    return np.log(10) * livetime * flux * energies * Veff / get_interaction_length(energies, cross_section_type = nuCrsScn) * dlogE


def get_exposure(energy, Veff, field_of_view=2*np.pi):
    """
    calculate exposure from effective volume

    Parameters
    ----------
    energy: float
        neutrino energy (needed to calculate interaction length)
    Veff: float
        effective volume
    field_of_view: float
        the field of view of the detector

    Returns
    float: exposure
    """
    return Veff / field_of_view / get_interaction_length(energy)

def get_integrated_exposure(exp_func, E_low, E_high):
    """
    calculates the integral int(E^-2 * exposure(E) dE)

    integration is performed in log space
    """

    def f(logE):
        E = 10 ** logE
        return exp_func(E) * np.log(E) / E

    from scipy import integrate
    i = integrate.quad(f, np.log10(E_low), np.log10(E_high))
    return i[0]

def get_fluence_limit(int_exp):
    """
    calculates the fluence limit for a integrated exposure
    """
    return 2.39/int_exp




if __name__=="__main__":  # this part of the code gets only executed it the script is directly called


    energy = 10**18 * units.eV
    veff = 2150 * units.km**3 * units.sr
    livetime = 5 *units.year

    print "Cross section: {} cm^2".format(cross_sections.get_nu_cross_section(energy, cross_section_type = 'ctw'))

    print "interaction length: {} km".format(get_interaction_length(energy, cross_section_type = 'ctw')/units.km)

    print "calculating flux limit for {time} years and Veff of {veff} km^3 sr".format(time=livetime/units.year,
                            veff = veff/ (units.km**3 * units.sr))
    print "Flux limit: {} GeV/(cm^2 s sr)".format(get_limit_e2_flux(energy,veff, livetime) / (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1))

    aeff = np.array([0.0032,0.033,0.43,3.1,21,68,167])* units.km**2 * units.sr
    energies = 10**np.array([18,18.5,19,19.5,20,20.5,21])* units.eV

    print "Flux limit: {} GeV/(cm^2 s sr)".format(energies**2*get_limit_from_aeff(energies,aeff,livetime) / (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1))










# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function
import numpy as np
from NuRadioMC.utilities import units
from NuRadioMC.utilities import inelasticities
from six import iterkeys, iteritems
from scipy import constants
from scipy.integrate import quad
from scipy.interpolate import interp1d
import scipy.interpolate as interpolate
from scipy.optimize import fsolve
from scipy.interpolate import RectBivariateSpline
import h5py
import os
import logging
logger = logging.getLogger("EventGen")
logging.basicConfig()

VERSION_MAJOR = 1
VERSION_MINOR = 1

HEADER = """
# all quantities are in the default NuRadioMC units (i.e., meters, radians and eV)
# all geometry quantities are in the NuRadioMC default local coordinate system:
#     coordinate origin is at the surface
#     x axis is towards Easting, y axis towards Northing, z axis upwards
#     zenith/theta angle is defined with respect to z axis, i.e. 0deg = upwards, 90deg = towards horizon, 180deg = downwards
#     azimuth/phi angle counting northwards from East
#
# the collumns are defined as follows
# 1. event id (integer)
# 2. neutrino flavor (integer) encoded as using PDG numbering scheme, particles have positive sign, anti-particles have negative sign, relevant for us are:
#       12: electron neutrino
#       14: muon neutrino
#       16: tau neutrino
# 3. energy of neutrino (double)
# 4. charge or neutral current interaction (string, one of ['cc', 'nc']
# 5./6./7. position of neutrino interaction vertex in cartesian coordinates (x, y, z) (in default NuRadioMC local coordinate system)
# 8. zenith/theta angle of neutrino direction (pointing to where it came from, i.e. opposite to the direction of propagation)
# 9. azimuth/phi angle of neutrino direction (pointing to where it came from, i.e. opposite to the direction of propagation)
# 10. inelasticity (the fraction of neutrino energy that goes into the hadronic part)
#
"""
# Mass energy equivalent of the tau lepton
tau_mass = constants.physical_constants['tau mass energy equivalent in MeV'][0] * units.MeV
# Lifetime of the tau (rest frame). Taken from PDG
tau_rest_lifetime = 290.3 * units.fs
density_ice = 0.9167 * units.g / units.cm ** 3
cspeed = constants.c * units.m / units.s

def load_input_hdf5(filename):
    """
    reads input file into memory

    Parameters
    ----------
    filename: string
        Name of the file

    Returns
    -------
    fin: dictionary
        Dictionary containing the elements in filename
    """
    h5fin = h5py.File(filename, 'r')
    fin = {}
    for key, value in iteritems(h5fin):
        fin[key] = np.array(value)
    h5fin.close()
    return fin

def create_interp(filename):
    """
    Creates RectBivariateSpline functions for interpolating the decay
    times and energies from filename

    Parameters
    ----------
    filename: string
        name of the hdf5 file containing the table

    Returns
    -------
    (interp_time, interp_energies): tuple of RectBivariateSpline functions
    """
    fin = load_input_hdf5(filename)

    log_time_bins = np.log10(fin['rest_times'])
    log_energy_bins = np.log10(fin['initial_energies'])

    f_time = RectBivariateSpline(log_time_bins, log_energy_bins, np.log10(fin['decay_times']) )
    f_energies = RectBivariateSpline(log_time_bins, log_energy_bins, np.log10(fin['decay_energies']) )

    def interp_time(time, energy):
        return 10**f_time(np.log10(time), np.log10(energy))

    def interp_energies(time, energy):
        return 10**f_energies(np.log10(time), np.log10(energy))

    return (interp_time, interp_energies)

def mean_energy_loss(energy):
    """
    Returns the mean energy loss of a tau per g/cm2 as a function of the tau energy
    This function is a linear (in log scale) approximation above 1 PeV to the
    curve found in https://doi.org/10.1016/j.astropartphys.2006.11.003

    Parameters
    ----------
    energy: float
       Tau energy

    Returns
    -------
    Energy loss per amount of matter (float)
    """
    E0 = 1*units.PeV
    if (energy > E0):
        b1 = 1.e-7 * units.cm ** 2 / units.g
        b2 = 1.8e-7 * units.cm ** 2 / units.g
        return b1*energy + b2*energy*np.log10(energy/E0)
    else:
        return 0.

def get_tau_decay_rest(energy):
    """
    Calculates the random tau decay time without time dilation
    """
    # The tau decay time is taken assuming an exponential decay
    # and applying the inverse transform method
    tau_decay_rest = -np.log(1 - np.random.uniform(0, 1)) * tau_rest_lifetime

    return tau_decay_rest

def get_tau_decay_time(energy):
    """
    Calculates the random tau decay time taking into account time dilation
    """

    gamma = energy / tau_mass  # tau_mass must be in natural units (c = 1)
    tau_decay_rest = get_tau_decay_rest(energy)
    tau_decay_time = gamma * tau_decay_rest

    return tau_decay_time

def get_decay_time_losses(energy, distmax, average=False, compare=False, user_time=None):
    """
    Calculates the decay time assuming photonuclear energy losses above
    1 PeV and using the quasi-continuous approximation.
    See https://doi.org/10.1016/j.astropartphys.2006.11.003 for details.

    Parameters
    ----------
    energy: float
        energy of the incident neutrino
    distmax: float
        maximum distance for which we calculate energy losses.
        It should be similar to the maximal dimension of the simulation volume.
    average: bool
        If False, a random decay time at rest is calculated
        If True, the tau mean lifetime at rest is used
    compare: bool
        If True, returns a tuple with the decay time with losses and without
        If False, only the decay time with losses is returned
    user_time: float
        If user_time is not None, the tau decay time in rest frame is taken as
        equal to user_time and the average flag is ignored.

    Returns
    -------
    decay_time: float
        Tau decay time with photonuclear losses
    energy_decay: float
        Tau energy at the time of decay
    decay_time_no_losses: float
        Tau decay time without losses
    """
    E0 = 1*units.PeV
    if (energy <= E0):
        #raise ValueError('Energy is equal to or less than 1 PeV. Returning decay time without energy loss.')
        if user_time is None:
            return get_tau_decay_time(energy), energy
        else:
            gamma = energy/tau_mass
            return gamma*user_time, energy

    # At these energies, we can use the speed of light as the tau speed
    timemax = distmax/cspeed
    Estep = energy/1000.

    times = [0.]
    timebreak = None
    energies = np.arange(energy, E0, -Estep)
    if (energies[-1] != E0):
        energies = np.append(energies, E0)

    # This function returns the inverse of the energy loss, needed for the
    # calculation of the ellapsed times
    def loss_int(E):
        return 1./mean_energy_loss(E)

    # We loop over the energies and integrate the inverse of the energy loss
    # so that we obtain the corresponding time at which the particle has a
    # given energy.
    for finalenergy in energies[1:]:

        # If the energy is less than 1 PeV, we stop.
        if (finalenergy < E0):
            timebreak = quad(loss_int, E0, energy)/density_ice/cspeed
            times.append(timebreak)
            break

        time = quad(loss_int, finalenergy, energy)[0]/density_ice/cspeed
        times.append(time)
        if (time > timemax):
            break

    energies = energies[0:len(times)]

    if user_time is not None:
        tau_decay_rest = user_time
    elif not average:
        tau_decay_rest = get_tau_decay_rest(energy)
    else:
        tau_decay_rest = tau_rest_lifetime

    # We use an interpolation for having the energies as a function of time
    energies_interp = interp1d(times, energies)

    # This function returns the Lorentz factor for a given time t
    def gamma(t):
        if timebreak is not None and t > timebreak:
            return E0/tau_mass
        elif (t > timemax):
            return energies[-1]/tau_mass
        elif (t > times[-1]):
            return E0/tau_mass
        elif (t < 0):
            return np.inf
        else:
            return energies_interp(t)/tau_mass

    # This function returns the inverse of the Lorentz factor at a time t
    def inv_gamma(t):
        return 1./gamma(t)

    # This function integrates the inverse of the Lorentz factor in order
    # to obtain the proper time for the tau between the times t0 and t1
    def proper_time(gamma_function, t0, t1):
        return quad(inv_gamma, t0, t1)[0]

    # This function returns the difference between the proper time of the tau
    # at a time t and its decay time (in the tau rest frame)
    def times_diff(t):
        return proper_time(gamma, 0, t)-tau_decay_rest

    # We obtain the decay time for the observer finding the roots for the
    # difference between the proper time and the decay time in the rest frame
    decay_time = fsolve(times_diff,1e3*units.ns)[0]
    energy_decay = gamma(decay_time)*tau_mass

    if not compare:
        return decay_time, energy_decay
    else:
        decay_time_no_losses = tau_decay_rest*gamma(0)
        return decay_time, decay_time_no_losses, energy_decay


def get_tau_speed(energy):
    """
    Calculates the speed of the tau lepton
    """

    gamma = energy / tau_mass
    if (gamma < 1):
        #raise ValueError('The energy is less than the tau mass. Returning zero speed')
        return 0
    beta = np.sqrt(1 - 1 / gamma ** 2)

    return beta * constants.c * units.m / units.s

def get_tau_decay_length(energy, distmax=0, table=None):
    """
    calculates the decay length of the tau

    Parameters
    ----------
    energy: float
       Tau energy
    distmax: float
    maximum distance for which we calculate energy losses.
    It should be similar to the maximal dimension of the simulation volume.
    table: RectBivariateSpline type function. See get_decay_time_tab.

    Returns
    -------
    decay_time, decay_energy: float, float
       Tau decay time and tau decay energy
    """

    if (energy <= 1*units.PeV):
        decay_time = get_tau_decay_time(energy)
        v = get_tau_speed(energy)
        return decay_time * v, energy
    else:
        if table is None:
            decay_time, decay_energy = get_decay_time_losses(energy, distmax)
        else:
            decay_time, decay_energy = get_decay_time_tab(table, energy)
        return decay_time * cspeed, decay_energy

def get_decay_time_tab(table, energy, time=None):
    """
    Calculates the decay time assuming photonuclear energy losses above
    1 PeV and using the quasi-continuous approximation.
    See https://doi.org/10.1016/j.astropartphys.2006.11.003 for details.
    This version uses tabulated histograms to speed up the computation.

    Parameters
    ----------
    table: tuple of 2 RectBivariateSpline type functions
        table[0](time,energy) must interpolate the decay time in lab frame
        table[1](time,energy) must interpolate the decay energy in lab frame
    energy: float
        energy of the incident neutrino
    time: float
        If time is not None, the tau decay time in rest frame is taken as
        equal to time. If time is None, a random time is drawn.

    Returns
    -------
    decay_time: float
        Tau decay time with photonuclear losses
    energy_decay: float
        Tau energy at the time of decay
    """

    if time is None:
        time = get_tau_decay_rest(energy)

    decay_time = table[0](time, energy)[0,0]
    decay_energy = table[1](time, energy)[0,0]

    return decay_time, decay_energy

def get_tau_decay_vertex(x, y, z, E, zenith, azimuth, distmax, table=None):
    """
     Let us assume that the tau has the same direction as the tau neutrino
     to calculate the vertex of the second shower

     Parameters
     ----------
     x: float
        x coordinate of the vertex position
     y: float
        y coordinate of the vertex position
     z: float
        z coordinate of the vertex position
     E: float
        Tau energy after neutrino interaction
     zenith: float
        Zenith arrival direction
     azimuth: float
        Azimuth arrival direction
     distmax: float
        maximum distance for which we calculate energy losses.
        It should be similar to the maximal dimension of the simulation volume.
     table: tuple of 2 RectBivariateSpline type functions

     Returns
     -------
     second_vertex_x: float
        x coordinate of the decay position
     second_vertex_y: float
        y coordinate of the decay position
     second_vertex_z: float
        z coordinate of the decay position
     decay_energy: float
        Tau energy at the moment of decay
    """
    L, decay_energy = get_tau_decay_length(E, distmax, table)
    second_vertex_x = L
    second_vertex_x *= np.sin(zenith) * np.cos(azimuth)
    second_vertex_x += x

    second_vertex_y = L
    second_vertex_y *= np.sin(zenith) * np.sin(azimuth)
    second_vertex_y += y

    second_vertex_z = L
    second_vertex_z *= np.cos(zenith)
    second_vertex_z += z
    return second_vertex_x, second_vertex_y, second_vertex_z, decay_energy

def get_tau_cascade_properties(tau_energy):
    """
    Given the energy of a decaying tau, calculates the properties of the
    resulting cascade.

    Parameters
    ----------
    tau_energy: float
       Tau energy at the moment of decay

    Returns
    -------
    cascade_energy: float
        The energy of the resulting cascade
    cascade_type: string
        Decay type: 'tau_had', 'tau_em', or 'tau_mu'
    """
    # TODO: calculate cascade energy
    # TODO: include the rest of the particles produced

    branch = inelasticities.random_tau_branch()
    products = inelasticities.inelasticity_tau_decay(tau_energy, branch)
    return products, branch



def write_events_to_hdf5(filename, data_sets, attributes, n_events_per_file=None,
                         start_file_id=0):
    """
    writes NuRadioMC input parameters to hdf5 file

    this function can automatically split the dataset up into multiple files for easy multiprocessing

    Parameters
    ----------
    filename: string
        the desired output filename (if multiple files are generated, a 'part000x' is appended to the filename
    data_sets: dict
        a dictionary with the data sets
    attributes: dict
        a dictionary containing the meta attributes
    n_events_per_file: int (optional, default None)
        the number of events per file
    additional_interactions: dict or None (default)
        a dictionary containing potential additional interactions, such as the second tau interaction vertex.
    """
    n_events = len(np.unique(data_sets['event_ids']))
    logger.info("saving {} events in total".format(n_events))
    total_number_of_events = attributes['n_events']
    
    if "start_event_id" not in attributes:
        attributes["start_event_id"] = 0  # backward compatibility
    
    if(n_events_per_file is None):
        n_events_per_file = n_events
    else:
        n_events_per_file = int(n_events_per_file)
    iFile = -1
    evt_id_first = data_sets['event_ids'][0]
    evt_id_last_previous = 0  # save the last event id of the previous file
    start_index = 0
    n_events_total = 0
    while True:
        iFile += 1
        filename2 = filename
        evt_ids_this_file = np.unique(data_sets['event_ids'])[iFile * n_events_per_file : (iFile + 1) * n_events_per_file]
        if(len(evt_ids_this_file) == 0):
            logger.info("no more events to write in file {}".format(iFile))
            break

        if((iFile > 0) or (n_events_per_file < n_events)):
            filename2 = filename + ".part{:04}".format(iFile + 1 + start_file_id)
        fout = h5py.File(filename2, 'w')
        fout.attrs['VERSION_MAJOR'] = VERSION_MAJOR
        fout.attrs['VERSION_MINOR'] = VERSION_MINOR
        fout.attrs['header'] = HEADER
        for key, value in attributes.iteritems():
            fout.attrs[key] = value
        fout.attrs['total_number_of_events'] = total_number_of_events

        evt_id_first = evt_ids_this_file[0]
        evt_id_last = evt_ids_this_file[-1]

        tmp = np.squeeze(np.argwhere(data_sets['event_ids'] == evt_id_last))  # set stop index such that last event is competely in file
        if(tmp.size == 1):
            stop_index = tmp + 1
        else:
            stop_index = tmp[-1] + 1
#         if(evt_id_last >= n_events):
#             evt_id_last = n_events
#             stop_index = len(data_sets['event_ids'])
#         else:
#             tmp = np.squeeze(np.argwhere(data_sets['event_ids'] > evt_id_last))  # set stop index such that last event is competely in file
#             if(tmp.size == 1):
#                 stop_index = tmp
#             else:
#                 stop_index = tmp[0]

        for key, value in data_sets.iteritems():
            fout[key] = value[start_index:stop_index]

        # determine the number of events in this file (which is NOT the same as the entries in the file)
        # case 1) this is not the last file -> number of events is difference between last event id of the current and previous file + 1
        # case 2) it is the last file -> total number of simulated events - last event id of previous file
        # case 3) it is the first file -> last event id + 1 - start_event_id
        # case 4) it is the first and last file -> total number of simulated events
        evt_ids_next_file = np.unique(data_sets['event_ids'])[(iFile + 1) * n_events_per_file : (iFile + 2) * n_events_per_file]
        n_events_this_file = None
        if(iFile == 0 and len(evt_ids_next_file) == 0):  # case 4
            n_events_this_file = total_number_of_events
        elif(len(evt_ids_next_file) == 0): # last file -> case 2
            n_events_this_file = total_number_of_events - evt_id_last_previous + attributes['start_event_id']
        elif(iFile == 0): # case 3
            n_events_this_file = evt_id_last - attributes['start_event_id'] + 1
        else: # case 1
            n_events_this_file = evt_id_last - evt_id_last_previous

        print('writing file {} with {} events (id {} - {}) and {} entries'.format(filename2, n_events_this_file, evt_id_first,
                                                                                  evt_id_last, stop_index - start_index))
        fout.attrs['n_events'] = n_events_this_file
        fout.close()
        n_events_total += n_events_this_file

        start_index = stop_index

        evt_id_last_previous = evt_id_last
        if(evt_id_last == n_events):  # break while loop if all events are saved
            break
    logger.info("wrote {} events in total".format(n_events_total))

def primary_energy_from_deposited(Edep, ccnc, flavor, inelasticity):
    """
    Calculates the primary energy of the neutrino from the deposited
    energy in the medium.

    Parameters
    ----------
    Edep: float
        deposited energy
    ccnc: string
        indicates 'nc', neutral current; 'cc', charged current
    flavor: int
        neutrino flavor
    inelasticity: float
        inelasticity of the interaction
    """

    if (ccnc == 'nc'):
        return Edep/inelasticity
    elif (ccnc == 'cc'):
        if (np.abs(flavor) == 12):
            return Edep
        elif (np.abs(flavor) == 14):
            return Edep/inelasticity
        elif (np.abs(flavor) == 16):
            return Edep/inelasticity # TODO: change this for taus
        
def ice_cube_nu_fit(energy, slope=-2.19, offset=1.01):
    # from https://doi.org/10.22323/1.301.1005
    # ApJ slope=-2.13, offset=0.9
    flux = 3 * offset * (energy / (100 * units.TeV))**slope * 1e-18 * \
        (units.GeV**-1 * units.cm**-2 * units.second**-1 * units.sr**-1)
    return flux

def get_GZK_1(energy):
    """
    model of (van Vliet et al., 2019, https://arxiv.org/abs/1901.01899v1) of the cosmogenic neutrino ﬂux
    for a source evolution parameter of m = 3.4, 
    a spectral index of the injection spectrum of α = 2.5, a cut-oﬀ rigidity of R = 100 EeV,
    and a proton fraction of 10% at E = 10^19.6 eV
    """
    E, J = np.loadtxt(os.path.join(os.path.abspath(os.path.dirname(__file__)),
                                     '../examples/Sensitivities/ReasonableNeutrinos1.txt'))
    E *= units.GeV
    J *= units.GeV * units.cm**-2 * units.s**-1 * units.sr**-1 / E**2
    get_flux = interpolate.interp1d(E, J, fill_value=0, bounds_error=False)
    return get_flux(energy)

def get_energy_from_flux(Emin, Emax, n_events, flux):
    """
    returns randomly distribution of energy according to a flux
    
    Parameters
    ----------
    Emin: float
        minumum energy
    Emax: float
        maximum energy
    n_event: int
        number of events to generate
    flux: function
        must return flux as function of energy in units of events per energy, time, solid angle and area
        
    Returns: array of energies
    """
    
    xx_edges = np.linspace(Emin, Emax, 10000000)
    xx = 0.5 * (xx_edges[1:] + xx_edges[:-1])
    yy = flux(xx)
    cum_values = np.zeros(xx_edges.shape)
    cum_values[1:] = np.cumsum(yy * np.diff(xx_edges))
    inv_cdf = interpolate.interp1d(cum_values, xx_edges)
    r = np.random.uniform(0, cum_values.max(), n_events)
    return inv_cdf(r)

def generate_eventlist_cylinder(filename, n_events, Emin, Emax,
                                fiducial_rmin, fiducial_rmax, fiducial_zmin, fiducial_zmax,
                                full_rmin=None, full_rmax=None, full_zmin=None, full_zmax=None,
                                thetamin=0.*units.rad, thetamax=np.pi*units.rad,
                                phimin=0.*units.rad, phimax=2*np.pi*units.rad,
                                start_event_id=1,
                                flavor=[12, -12, 14, -14, 16, -16],
                                n_events_per_file=None,
                                spectrum='log_uniform',
                                add_tau_second_bang=False,
                                tabulated_taus=True,
                                deposited=False,
                                start_file_id=0):
    """
    Event generator

    Generates neutrino interactions, i.e., vertex positions, neutrino directions,
    neutrino flavor, charged currend/neutral current and inelastiviy distributions.
    All events are saved in an hdf5 file.

    Parameters
    ----------
    filename: string
        the output filename of the hdf5 file
    n_events: int
        number of events to generate
    Emin: float
        the minimum neutrino energy (energies are randomly chosen assuming a
        uniform distribution in the logarithm of the energy)
    Emax: float
        the maximum neutrino energy (energies are randomly chosen assuming a
        uniform distribution in the logarithm of the energy)

    fiducial_rmin: float
        lower r coordinate of fiducial volume (the fiducial volume needs to be chosen large enough such that no events outside of it will trigger)
    fiducial_rmax: float
        upper r coordinate of fiducial volume (the fiducial volume needs to be chosen large enough such that no events outside of it will trigger)
    fiducial_zmin: float
        lower z coordinate of fiducial volume (the fiducial volume needs to be chosen large enough such that no events outside of it will trigger)
    fiducial_zmax: float
        upper z coordinate of fiducial volume (the fiducial volume needs to be chosen large enough such that no events outside of it will trigger)
    full_rmin: float (default None)
        lower r coordinate of simulated volume (if None it is set to 1/3 of the fiducial volume, if second vertices are not activated it is set to the fiducial volume)
    full_rmax: float (default None)
        upper r coordinate of simulated volume (if None it is set to 5x the fiducial volume, if second vertices are not activated it is set to the fiducial volume)
    full_zmin: float (default None)
        lower z coordinate of simulated volume (if None it is set to 1/3 of the fiducial volume, if second vertices are not activated it is set to the fiducial volume)
    full_zmax: float (default None)
        upper z coordinate of simulated volume (if None it is set to 5x the fiducial volume, if second vertices are not activated it is set to the fiducial volume)
    thetamin: float
        lower zenith angle for neutrino arrival direction
    thetamax: float
        upper zenith angle for neutrino arrival direction
    phimin: float
        lower azimuth angle for neutrino arrival direction
    phimax: float
         upper azimuth angle for neutrino arrival direction
    start_event: int
        default: 1
        event number of first event
    flavor: array of ints
        default: [12, -12, 14, -14, 16, -16]
        specify which neutrino flavors to generate. A uniform distribution of
        all specified flavors is assumed.
        The neutrino flavor (integer) encoded as using PDF numbering scheme,
        particles have positive sign, anti-particles have negative sign,
        relevant for us are:
        * 12: electron neutrino
        * 14: muon neutrino
        * 16: tau neutrino
    n_events_per_file: int or None
        the maximum number of events per output files. Default is None, which
        means that all events are saved in one file. If 'n_events_per_file' is
        smaller than 'n_events' the event list is split up into multiple files.
        This is useful to split up the computing on multiple cores.
    spectrum: string
        defines the probability distribution for which the neutrino energies are generated
        * 'log_uniform': uniformly distributed in the logarithm of energy
        * 'E-?': E to the -? spectrum where ? can be any float
        * 'IceCube-nu-2017': astrophysical neutrino flux measured with IceCube muon sample (https://doi.org/10.22323/1.301.1005)
        * 'GZK-1': GZK neutrino flux model from van Vliet et al., 2019, https://arxiv.org/abs/1901.01899v1 for
                   10% proton fraction (see get_GZK_1 function for details)
        * 'GZK-1+IceCube-nu-2017': a combination of the cosmogenic (GZK-1) and astrophysical (IceCube nu 2017) flux
    add_tau_second_bang: bool
        if True simulate second vertices from tau decays
    tabulated_taus: bool
        if True the tau decay properties are taken from a table
    deposited: bool
        If True, generate deposited energies instead of primary neutrino energies
    start_file_id: int (default 0)
        in case the data set is distributed over several files, this number specifies the id of the first file
        (useful if an existing data set is extended)
    """
    attributes = {}
    n_events = int(n_events)
    attributes['n_events'] = n_events
    attributes['start_event_id'] = start_event_id

    attributes['fiducial_rmin'] = fiducial_rmin
    attributes['fiducial_rmax'] = fiducial_rmax
    attributes['fiducial_zmin'] = fiducial_zmin
    attributes['fiducial_zmax'] = fiducial_zmax

    if(full_rmin is None):
        if(add_tau_second_bang):
            full_rmin = fiducial_rmin / 3.
        else:
            full_rmin = fiducial_rmin
    if(full_rmax is None):
        if(add_tau_second_bang):
            full_rmax = fiducial_rmax * 5.
        else:
            full_rmax = fiducial_rmax
    if(full_zmin is None):
        if(add_tau_second_bang):
            full_zmin = fiducial_zmin * 5.
        else:
            full_zmin = fiducial_zmin
    if(full_zmax is None):
        if(add_tau_second_bang):
            full_zmax = fiducial_zmax / 3.
        else:
            full_zmax = fiducial_zmax

    attributes['rmin'] = full_rmin
    attributes['rmax'] = full_rmax
    attributes['zmin'] = full_zmin
    attributes['zmax'] = full_zmax
    attributes['flavors'] = flavor
    attributes['Emin'] = Emin
    attributes['Emax'] = Emax
    attributes['thetamin'] = thetamin
    attributes['thetamax'] = thetamax
    attributes['phimin'] = phimin
    attributes['phimax'] = phimax
    attributes['deposited'] = deposited

    data_sets = {}
    # generate neutrino vertices randomly
    data_sets["azimuths"] = np.random.uniform(phimin, phimax, n_events)
    u = np.random.uniform(np.cos(thetamax), np.cos(thetamin), n_events)
    data_sets["zeniths"] = np.arccos(u)  # generates distribution that is uniform in cos(theta)

    rr_full = np.random.triangular(full_rmin, full_rmax, full_rmax, n_events)
    data_sets["xx"] = rr_full * np.cos(data_sets["azimuths"])
    data_sets["yy"] = rr_full * np.sin(data_sets["azimuths"])
    data_sets["zz"] = np.random.uniform(full_zmin, full_zmax, n_events)
    fmask = (rr_full >= fiducial_rmin) & (rr_full <= fiducial_rmax) & (data_sets["zz"] >= fiducial_zmin) & (data_sets["zz"] <= fiducial_zmax)  # fiducial volume mask

    data_sets["event_ids"] = np.arange(n_events) + start_event_id
    data_sets["n_interaction"] = np.ones(n_events, dtype=np.int)

    # generate neutrino flavors randomly
    data_sets["flavors"] = np.array([flavor[i] for i in np.random.randint(0, high=len(flavor), size=n_events)])
    """
    #from AraSim nue:nueb:numu:numub:nutau:nutaub = 0.78: 0.22: 0.61: 0.39: 0.61: 0.39
    flaRnd = np.random.uniform(0., 3., n_events)
    flavors = np.ones(n_events, dtype = np.int64)
    for i, r in enumerate(flaRnd):
        if (r <= 0.78):
            flavors[i] = flavor[0]
        elif (r <= 1.0):
            flavors[i] = flavor[1]
        elif (r <= 1.61):
            flavors[i] = flavor[2]
        elif (r <= 2.0):
            flavors[i] = flavor[3]
        elif (r <= 2.61):
            flavors[i] = flavor[4]
        else:
            flavors[i] = flavor[5]
    """
    # generate energies randomly
    if(spectrum == 'log_uniform'):
        data_sets["energies"] = 10 ** np.random.uniform(np.log10(Emin), np.log10(Emax), n_events)
    elif(spectrum.startswith("E-")):  # enerate an E^gamma spectrum.  
        gamma = float(spectrum[1:])
        gamma += 1
        Nmin = (Emin)**gamma
        Nmax = (Emax)**gamma
        def get_inverse_spectrum(N, gamma):
            return np.exp(np.log(N)/gamma)
        data_sets["energies"] = get_inverse_spectrum(np.random.uniform(Nmax, Nmin, size=n_events), gamma)
    elif(spectrum == "GZK-1"):
        """
        model of (van Vliet et al., 2019, https://arxiv.org/abs/1901.01899v1) of the cosmogenic neutrino ﬂux
        for a source evolution parameter of m = 3.4, 
        a spectral index of the injection spectrum of α = 2.5, a cut-oﬀ rigidity of R = 100 EeV,
        and a proton fraction of 10% at E = 10^19.6 eV
        """
        data_sets["energies"] = get_energy_from_flux(Emin, Emax, n_events, get_GZK_1)
    elif(spectrum == "IceCube-nu-2017"):
        data_sets["energies"] = get_energy_from_flux(Emin, Emax, n_events, ice_cube_nu_fit)
    elif(spectrum == "GZK-1+IceCube-nu-2017"):
        def J(E):
            return ice_cube_nu_fit(E) + get_GZK_1(E)
        data_sets["energies"] = get_energy_from_flux(Emin, Emax, n_events, J)
    else:
        logger.error("spectrum {} not implemented".format(spectrum))
        raise NotImplementedError("spectrum {} not implemented".format(spectrum))

    # generate charged/neutral current randomly
    data_sets["interaction_type"] = inelasticities.get_ccnc(n_events)

    # generate inelasticity
    data_sets["inelasticity"] = inelasticities.get_neutrino_inelasticity(n_events)
    """
    #from AraSim
    epsilon = np.log10(energies / 1e9)
    inelasticity = pickY(flavors, ccncs, epsilon)
    """
    if deposited:
        data_sets["energies"] = [primary_energy_from_deposited(Edep, ccnc, flavor, inelasticity) \
                                for Edep, ccnc, flavor, inelasticity in \
                                zip(data_sets["energies"], data_sets["interaction_type"], \
                                data_sets["flavors"], data_sets["inelasticity"])]
        data_sets["energies"] = np.array(data_sets["energies"])

    data_sets_fiducial = {}

    if not add_tau_second_bang:
        # save only events with interactions in fiducial volume
        for key, value in iteritems(data_sets):
            data_sets_fiducial[key] = value[fmask]

    else:
        # Initialising data_sets_fiducial with empty values
        for key, value in iteritems(data_sets):
            data_sets_fiducial[key] = []

        if tabulated_taus:
            cdir = os.path.dirname(__file__)
            table = create_interp(os.path.join(cdir, 'decay_library.hdf5'))
        else:
            table = None

        mask = (data_sets["interaction_type"] == 'cc') & (np.abs(data_sets["flavors"]) == 16)
        logger.info("{} taus are created in nu tau interactions -> checking if tau decays in fiducial volume".format(np.sum(mask)))
        n_taus = 0
        for event_id in data_sets["event_ids"]:
            iE = event_id - start_event_id

            first_inserted = False

            x = data_sets['xx'][iE]
            y = data_sets['yy'][iE]
            z = data_sets['zz'][iE]
            r = (x ** 2 + y ** 2)**0.5

            # Appending event if it interacts within the fiducial volume
            if ( r >= fiducial_rmin and r <= fiducial_rmax ):
                if ( z >= fiducial_zmin and z <= fiducial_zmax ):

                    for key in iterkeys(data_sets):
                        data_sets_fiducial[key].append(data_sets[key][iE])

                    first_inserted = True

            if (data_sets["interaction_type"][iE] == 'cc' and np.abs(data_sets["flavors"][iE]) == 16):

                Etau = (1 - data_sets["inelasticity"][iE]) * data_sets["energies"][iE]

                # first calculate if tau decay is still in our fiducial volume
                x, y, z, decay_energy = get_tau_decay_vertex(data_sets["xx"][iE], data_sets["yy"][iE], data_sets["zz"][iE],
                                               Etau, data_sets["zeniths"][iE], data_sets["azimuths"][iE],
                                               np.sqrt(4*(full_rmax-full_rmin)**2+(full_zmax-full_zmin)**2), table=table)

                r = (x ** 2 + y ** 2)**0.5
                if( r >= fiducial_rmin and r <= fiducial_rmax ):
                    if(z >= fiducial_zmin and z <= fiducial_zmax):  # z coordinate is negative
                        # the tau decay is in our fiducial volume

                        n_taus += 1
                        # If the tau decays in the fiducial volume but the parent neutrino does not
                        # interact there, we add it to know its properties.
                        if not first_inserted:
                            copies = 2
                        else:
                            copies = 1

                        for icopy in range(copies):
                            for key in iterkeys(data_sets):
                                data_sets_fiducial[key].append(data_sets[key][iE])

                        y_cascade, cascade_type = get_tau_cascade_properties(decay_energy)
                        data_sets_fiducial['n_interaction'][-1] = 2 # specify that new event is a second interaction
                        data_sets_fiducial['energies'][-1] = decay_energy
                        data_sets_fiducial['inelasticity'][-1] = y_cascade
                        data_sets_fiducial['interaction_type'][-1] = cascade_type
                        # TODO: take care of the tau_mu
                        data_sets_fiducial['xx'][-1] = x
                        data_sets_fiducial['yy'][-1] = y
                        data_sets_fiducial['zz'][-1] = z

                        # set flavor to tau
                        data_sets_fiducial['flavors'][-1] = 15 * np.sign(data_sets['flavors'][iE])  # keep particle/anti particle nature
        logger.info("added {} tau decays to the event list".format(n_taus))

        # Transforming every array into a numpy array and copying it back to
        # data_sets_fiducial
        for key in iterkeys(data_sets):
            data_sets_fiducial[key] = np.array(data_sets_fiducial[key])

    write_events_to_hdf5(filename, data_sets_fiducial, attributes, n_events_per_file=n_events_per_file, start_file_id=start_file_id)


def split_hdf5_input_file(input_filename, output_filename, number_of_events_per_file):
    """
    splits up an existing hdf5 file into multiple subfiles

    Parameters
    ----------
    input_filename: string
        the input filename
    output_filename: string
        the desired output filename (if multiple files are generated, a 'part000x' is appended to the filename
    n_events_per_file: int (optional, default None)
        the number of events per file
    """
    fin = h5py.File(input_filename, 'r')
    data_sets = {}
    attributes = {}
    for key, value in fin.items():
        if isinstance(value, h5py.Dataset):  # the loop is also over potential subgroupu that we don't want to consider here
            data_sets[key] = np.array(value)
    for key, value in fin.attrs.items():
        attributes[key] = value

    fin.close()

    write_events_to_hdf5(output_filename, data_sets, attributes, n_events_per_file=number_of_events_per_file)



from subprocess import Popen, PIPE
import os
import re
import logging
logger = logging.getLogger("utilities.version")
logging.basicConfig()


def get_git_commit_hash(path):
    gitproc = Popen(['git', 'show'], stdout = PIPE, cwd=path)
    (stdout, stderr) = gitproc.communicate()
    try:
        h = stdout.split('\n')[0].split()[1]
        check = re.compile(r"^[a-f0-9]{40}(:.+)?$", re.IGNORECASE)
        if(not check.match(h)):
            logging.error("NuRadioMC version could not be determined, returning None")
            return None
    except:
        return None
    return h

def get_NuRadioMC_commit_hash():
    """
    returns the hash of the current commit of the NuRadioMC git repository
    """
    import NuRadioMC
    path = os.path.dirname(NuRadioMC.__file__)
    return get_git_commit_hash(path)

def get_NuRadioReco_commit_hash():
    """
    returns the hash of the current commit of the NuRadioReco git repository
    """
    import NuRadioReco
    path = os.path.dirname(NuRadioReco.__file__)
    return get_git_commit_hash(path)


#!/usr/bin/env python
from __future__ import absolute_import, division, print_function
import sys
import h5py
import numpy as np
from numpy import testing
import argparse
from NuRadioReco.utilities import units
import logging


file1 = sys.argv[1]
file2 = sys.argv[2]

fin1 = h5py.File(file1, 'r')
fin2 = h5py.File(file2, 'r')

# Test all attributes in file

attributes = [u'trigger_names',
 u'antenna_positions',
 u'Tnoise',
 u'Vrms',
 u'dt',
 u'bandwidth',
 u'n_samples',
 u'thetamin',
 u'zmax',
 u'zmin',
 u'thetamax',
 u'header',
 u'fiducial_zmax',
 u'fiducial_zmin',
 u'flavors',
 u'rmin',
 u'total_number_of_events',
 u'deposited',
 u'phimax',
 u'phimin',
 u'Emin',
 u'rmax',
 u'fiducial_rmax',
 u'Emax',
 u'fiducial_rmin',
 u'n_events']
for key in attributes:
    testing.assert_equal(fin1.attrs[key], fin2.attrs[key])

#Test all keys in file

keys = [u'SNRs',
    u'azimuths',
    u'energies',
    u'event_ids',
    u'flavors',
    u'inelasticity',
    u'interaction_type',
    u'launch_vectors',
    u'maximum_amplitudes',
    u'maximum_amplitudes_envelope',
    u'multiple_triggers',
    u'n_interaction',
    u'polarization',
    u'ray_tracing_C0',
    u'ray_tracing_C1',
    u'ray_tracing_solution_type',
    u'receive_vectors',
    u'travel_distances',
    u'travel_times',
    u'triggered',
    u'weights',
    u'xx',
    u'yy',
    u'zeniths',
    u'zz']

for key in keys:
    testing.assert_equal(np.array(fin1[key]), np.array(fin2[key]))


# keys2 = [u'SNRs',
#  u'launch_vectors',
#  u'maximum_amplitudes',
#  u'maximum_amplitudes_envelope',
#  u'multiple_triggers',
#  u'polarization',
#  u'ray_tracing_C0',
#  u'ray_tracing_C1',
#  u'ray_tracing_solution_type',
#  u'receive_vectors',
#  u'travel_distances',
#  u'travel_times',
#  u'triggered']
#
# for key in keys2:
#     testing.assert_equal(np.array(fin1['station_101'][key]), np.array(fin2['station_101'][key]))


print("if no error occured, previous results are exactly reproduced.")






import matplotlib.pyplot as plt
import numpy as np
import time
from NuRadioMC.SignalProp import analyticraytraycing as ray
from NuRadioMC.utilities import units, medium
import logging
from radiotools import helper as hp
from radiotools import plthelpers as php
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('raytracing')
# ray.cpp_available=False

x1 = np.array([478., 0., -149.])*units.m
x2 = np.array([635., 0., -5.])*units.m  # direct ray solution
x3 = np.array([1000., 0., -90.])*units.m  # refracted/reflected ray solution
x4 = np.array([700., 0., -149.])*units.m  # refracted/reflected ray solution
x5 = np.array([1000., 0., -5.])*units.m  # no solution

receive_vectors = np.zeros((4, 2, 3)) * np.nan
ray_tracing_C0 = np.zeros((4, 2)) * np.nan
ray_tracing_C1 = np.zeros((4, 2)) * np.nan
ray_tracing_solution_type = np.zeros((4, 2), dtype=np.int) * np.nan
travel_times = np.zeros((4, 2)) * np.nan
travel_distances = np.zeros((4, 2)) * np.nan

ice = medium.southpole_simple()

lss = ['-', '--', ':']
colors = ['b','g','r']
fig, ax = plt.subplots(1, 1)
ax.plot(x1[0], x1[2], 'ko')
for i, x in enumerate([x2, x3, x4, x5]):
    print('finding solutions for ', x)
    r = ray.ray_tracing(x1,x,ice)
    r.find_solutions()
    if(r.has_solution()):
        for iS in range(r.get_number_of_solutions()):
            ray_tracing_C0[i, iS] = r.get_results()[iS]['C0']
            ray_tracing_solution_type[i, iS] = r.get_solution_type(iS)
            print "     Solution %d, Type %d: "%(iS,ray_tracing_solution_type[i, iS])
            R = r.get_path_length(iS)  # calculate path length
            T = r.get_travel_time(iS)  # calculate travel time
            print "     Ray Distance %.3f and Travel Time %.3f"%(R/units.m,T/units.ns)
            receive_vector = r.get_receive_vector(iS)
            receive_vectors[i, iS] = receive_vector
            zenith, azimuth = hp.cartesian_to_spherical(*receive_vector)
            print "     Receiving Zenith %.3f and Azimuth %.3f "%(zenith/units.deg, azimuth/units.deg)

            #to readout the actual trace, we have to flatten to 2D
            dX = x - x1
            dPhi = -np.arctan2(dX[1],dX[0])
            c,s = np.cos(dPhi), np.sin(dPhi)
            R = np.array(((c,-s,0),(s,c,0),(0,0,1)))
            X1r = x1
            X2r = np.dot(R, x-x1) + x1
            x1_2d = np.array([X1r[0],X1r[2]])
            x2_2d = np.array([X2r[0],X2r[2]])
            r_2d = ray.ray_tracing_2D(ice)
            yy,zz = r_2d.get_path(x1_2d,x2_2d,ray_tracing_C0[i, iS])
            ax.plot(yy, zz, '{}'.format(php.get_color_linestyle(i)), label='{} C0 = {:.4f}'.format(ray_tracing_solution_type[i,iS], ray_tracing_C0[i,iS]))
            ax.plot(x2_2d[0], x2_2d[1], '{}{}-'.format('d',php.get_color(i)))

ax.legend()
ax.set_xlabel("y [m]")
ax.set_ylabel("z [m]")
fig.tight_layout()
fig.savefig('example.png')



import numpy as np

"""
A wrapper around the numpy fft routines to achive a coherent normalization of the fft
As we have real valued data in the time domain, we use the 'real ffts' that omit the negative frequencies in Fourier 
space. To account for the missing power in the frequency domain, we multiply the frequency spectrum by sqrt(2), 
and divide the iFFT with 1/sqrt(2) accordingly. Then, a calculation of the power leads the same result in 
the time and frequency domain, i.e.
np.sum(trace**2) * dt = np.sum(spectrum**2) * df
"""


def time2freq(trace):
    """
    performs forward FFT with correct normalization that conserves the power
    """
    return np.fft.rfft(trace, axis=-1) * 2 ** 0.5  # an additional sqrt(2) is added because negative frequencies are omitted.


def freq2time(spectrum):
    """
    performs backward FFT with correct normalization that conserves the power
    """
    return np.fft.irfft(spectrum, axis=-1) / 2 ** 0.5



from __future__ import absolute_import, division, print_function
import argparse
# import detector simulation modules
import NuRadioReco.modules.efieldToVoltageConverter
import NuRadioReco.modules.trigger.highLowThreshold
import NuRadioReco.modules.trigger.simpleThreshold
import NuRadioReco.modules.channelResampler
import NuRadioReco.modules.channelBandPassFilter
import NuRadioReco.modules.channelGenericNoiseAdder
from NuRadioReco.utilities import units
from NuRadioMC.simulation import simulation2 as simulation
import logging
logging.basicConfig(level=logging.WARNING)
logger = logging.getLogger("runstrawman")

# initialize detector sim modules
efieldToVoltageConverter = NuRadioReco.modules.efieldToVoltageConverter.efieldToVoltageConverter()
efieldToVoltageConverter.begin()
simpleThreshold = NuRadioReco.modules.trigger.simpleThreshold.triggerSimulator()
highLowThreshold = NuRadioReco.modules.trigger.highLowThreshold.triggerSimulator()
channelResampler = NuRadioReco.modules.channelResampler.channelResampler()
channelBandPassFilter = NuRadioReco.modules.channelBandPassFilter.channelBandPassFilter()
channelGenericNoiseAdder = NuRadioReco.modules.channelGenericNoiseAdder.channelGenericNoiseAdder()

class mySimulation(simulation.simulation):


    def _detector_simulation(self):
        # start detector simulation
        if(bool(self._cfg['signal']['zerosignal'])):
            self._increase_signal(None, 0)
        
        efieldToVoltageConverter.run(self._evt, self._station, self._det)  # convolve efield with antenna pattern
        # downsample trace to internal simulation sampling rate (the efieldToVoltageConverter upsamples the trace to
        # 20 GHz by default to achive a good time resolution when the two signals from the two signal paths are added) 
        channelResampler.run(self._evt, self._station, self._det, sampling_rate=1. / self._dt)
        
        if bool(self._cfg['noise']):
            Vrms = self._Vrms / (self._bandwidth /( 2.5 * units.GHz))** 0.5  # normalize noise level to the bandwidth its generated for
            channelGenericNoiseAdder.run(self._evt, self._station, self._det, amplitude=Vrms, min_freq=0 * units.MHz,
                                         max_freq=2.5 * units.GHz, type='rayleigh')
        
        # bandpass filter trace, the upper bound is higher then the sampling rate which makes it just a highpass filter
        channelBandPassFilter.run(self._evt, self._station, self._det, passband=[80 * units.MHz, 1000 * units.GHz],
                                  filter_type='butter', order=2)
        channelBandPassFilter.run(self._evt, self._station, self._det, passband=[0, 500 * units.MHz],
                                  filter_type='butter', order=10)
        # first run a simple threshold trigger
        simpleThreshold.run(self._evt, self._station, self._det,
                             threshold=3 * self._Vrms,
                             triggered_channels=None,  # run trigger on all channels
                             number_concidences=1,
                             trigger_name='simple_threshold')  # the name of the trigger
        
        # run a high/low trigger on the 4 downward pointing LPDAs
        highLowThreshold.run(self._evt, self._station, self._det,
                                    threshold_high=4 * self._Vrms,
                                    threshold_low=-4 * self._Vrms,
                                    triggered_channels=[0, 1, 2, 3],  # select the LPDA channels
                                    number_concidences=2,  # 2/4 majority logic
                                    trigger_name='LPDA_2of4_4.1sigma',
                                    set_not_triggered=(not self._station.has_triggered("simple_threshold"))) # calculate more time consuming ARIANNA trigger only if station passes simple trigger
        
        # run a high/low trigger on the 4 surface dipoles 
        highLowThreshold.run(self._evt, self._station, self._det,
                                    threshold_high=3 * self._Vrms,
                                    threshold_low=-3 * self._Vrms,
                                    triggered_channels=[4, 5, 6, 7], # select the bicone channels
                                    number_concidences=4, # 4/4 majority logic
                                    trigger_name='surface_dipoles_4of4_3sigma',
                                    set_not_triggered=(not self._station.has_triggered("simple_threshold"))) # calculate more time consuming ARIANNA trigger only if station passes simple trigger
        
        # downsample trace back to detector sampling rate
        channelResampler.run(self._evt, self._station, self._det, sampling_rate=self._sampling_rate_detector)


parser = argparse.ArgumentParser(description='Run NuRadioMC simulation')
parser.add_argument('inputfilename', type=str,
                    help='path to NuRadioMC input event list')
parser.add_argument('detectordescription', type=str,
                    help='path to file containing the detector description')
parser.add_argument('config', type=str,
                    help='NuRadioMC yaml config file')
parser.add_argument('outputfilename', type=str,
                    help='hdf5 output filename')
parser.add_argument('outputfilenameNuRadioReco', type=str, nargs='?', default=None,
                    help='outputfilename of NuRadioReco detector sim file')
args = parser.parse_args()

sim = mySimulation(eventlist=args.inputfilename,
                            outputfilename=args.outputfilename,
                            detectorfile=args.detectordescription,
                            outputfilenameNuRadioReco=args.outputfilenameNuRadioReco,
                            config_file=args.config)
sim.run()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import division, print_function
import numpy as np
from NuRadioReco.utilities import units
from scipy import interpolate as intp
from scipy import integrate as int
from scipy import constants
from matplotlib import pyplot as plt
from radiotools import coordinatesystems as cstrafo
import os
import copy
import pickle
import logging
logger = logging.getLogger("SignalGen.ARZ")
logging.basicConfig()

######################
######################
# This code is based on "J. Alvarez-Muniz, P. Hansen, A. Romero-Wolf, E. Zas in preparation" which is an extension of
# Alvarez-Muñiz, J., Romero-Wolf, A., & Zas, E. (2011). Practical and accurate calculations of Askaryan radiation. Physical Review D - Particles, Fields, Gravitation and Cosmology, 84(10). https://doi.org/10.1103/PhysRevD.84.103003
# to hadronic showers. Thanks a lot to Jaime Alvarez-Muñiz for providing us with this unpublished work!
#####################
#####################

# define constants
# x0 = 36.08 * units.g / units.cm**2  # radiation length g cm^-2
rho = 0.924 * units.g / units.cm ** 3  # density g cm^-3
xmu = 12.566370e-7 * units.newton / units.ampere ** 2
c = 2.99792458e8 * units.m / units.s
# e = 1.602177e-19 * units.coulomb


def thetaprime_to_theta(thetaprime, xmax, R):
    """
    convertes a viewing angle relative to the shower maximum to a viewing angle relative to the start of the shower. 
    """
    L = xmax / rho
    return thetaprime - np.arcsin((L * np.sin(np.pi - thetaprime)) / R)


def theta_to_thetaprime(theta, xmax, R):
    """
    converts a viewing angle relative to the start of the shower to a viewing angle relative to the shower maximum
    """
    L = xmax / rho
    b = R * np.sin(theta)
    a = R * np.cos(theta) - L
    return np.arctan2(b, a)


class ARZ(object):
    __instance = None

    def __new__(cls, seed=1234, interp_factor=1, interp_factor2=100, library=None):
        if ARZ.__instance is None:
            ARZ.__instance = object.__new__(cls, seed, interp_factor, interp_factor2, library)
        return ARZ.__instance

    def __init__(self, seed=1234, interp_factor=1, interp_factor2=100, library=None):
        logger.warning("setting seed to {}".format(seed, interp_factor))
        np.random.seed(seed)
        self._interp_factor = interp_factor
        self._interp_factor2 = interp_factor2
        self._random_numbers = {}
        self._version = (1, 2)
        # # load shower library into memory
        if(library is None):
            library = os.path.join(os.path.dirname(__file__), "shower_library/library_v{:d}.{:d}.pkl".format(*self._version))
        else:
            if(not os.path.exists(library)):
                logger.error("user specified shower library {} not found.".format(library))
                raise FileNotFoundError("user specified shower library {} not found.".format(library))
        self.__check_and_get_library()
        
        with open(library) as fin:
            logger.warning("loading shower library into memory")
            self._library = pickle.load(fin)
            
    def __check_and_get_library(self):
        """
        checks if shower library exists and is up to date by comparing the sha1sum. If the library does not exist
        or changes on the server, a new library will be downloaded. 
        """
        path = os.path.join(os.path.dirname(__file__), "shower_library/library_v{:d}.{:d}.pkl".format(*self._version))
        
        download_file = False
        if(not os.path.exists(path)):
            logger.warning("shower library version {} does not exist on the local file system yet. It will be downloaded to {}".format(self._version, path))
            download_file = True
    
        if(os.path.exists(path)):
            BUF_SIZE = 65536 * 2 ** 4  # lets read stuff in 64kb chunks!
            import hashlib
            import json
            sha1 = hashlib.sha1()
            with open(path, 'rb') as f:
                while True:
                    data = f.read(BUF_SIZE)
                    if not data:
                        break
                    sha1.update(data)
    
            shower_directory = os.path.join(os.path.dirname(__file__), "shower_library/")
            with open(os.path.join(shower_directory, 'shower_lib_hash.json'), 'r') as fin:
                lib_hashs = json.load(fin)
                if("{:d}.{:d}".format(*self._version) in lib_hashs.keys()):
                    if(sha1.hexdigest() != lib_hashs["{:d}.{:d}".format(*self._version)]):
                        logger.warning("shower library {} has changed on the server. downloading newest version...".format(self._version))
                        download_file = True
                else:
                    logger.warning("no hash sum of {} available, skipping up-to-date check".format(os.path.basename(path)))            
        if not download_file:
            return True
        else:
            import requests
            URL = 'http://arianna.ps.uci.edu/~arianna/data/ce_shower_library/library_v{:d}.{:d}.pkl'.format(*self._version)
    
            logger.info("downloading shower library {} from {}. This can take a while...".format(self._version, URL))
            r = requests.get(URL)
            if (r.status_code != requests.codes.ok):
                logger.error("error in download of antenna model")
                raise IOError("error in download of antenna model")
            with open(path, "wb") as code:
                code.write(r.content)
            logger.info("...download finished.")
        
    def set_seed(self, seed):
        """
        allow to set a new random seed
        """
        np.random.seed(seed)
        
    def set_interpolation_factor(self, interp_factor):
        """
        set interpolation factor of charge-excess profiles
        """
        self._interp_factor = interp_factor
        
    def set_interpolation_factor2(self, interp_factor):
        """
        set interpolation factor around peak of form factor
        """
        self._interp_factor2 = interp_factor

    def get_time_trace(self, shower_energy, theta, N, dt, shower_type, n_index, R, shift_for_xmax=False,
                       same_shower=False, iN=None, output_mode='trace', theta_reference='X0'):
        """
        calculates the electric-field Askaryan pulse from a charge-excess profile
        
        Parameters
        ----------
        shower_energy: float
            the energy of the shower
        theta: float
            viewing angle, i.e., the angle between shower axis and launch angle of the signal (the ray path)
        N: int
            number of samples in the time domain
        dt: float
            size of one time bin in units of time
        profile_depth: array of floats
            shower depth values of the charge excess profile
        profile_ce: array of floats
            charge-excess values of the charge excess profile
        shower_type: string (default "HAD")
            type of shower, either "HAD" (hadronic), "EM" (electromagnetic) or "TAU" (tau lepton induced)
        n_index: float (default 1.78)
            index of refraction where the shower development takes place
        R: float (default 1km)
            observation distance, the signal amplitude will be scaled according to 1/R
        interp_factor: int (default 10)
            interpolation factor of charge-excess profile. Results in a more precise numerical integration which might be beneficial 
            for small vertex distances but also slows down the calculation proportional to the interpolation factor. 
        shift_for_xmax: bool (default True)
            if True the observer position is placed relative to the position of the shower maximum, if False it is placed 
            with respect to (0,0,0) which is the start of the charge-excess profile
        same_shower: bool (default False)
            if False, for each request a new random shower realization is choosen. 
            if True, the shower from the last request of the same shower type is used. This is needed to get the Askaryan
            signal for both ray tracing solutions from the same shower. 
        iN: int or None (default None)
            specify shower number
        output_mode: string
            * 'trace' (default): return only the electric field trace
            * 'Xmax': return trace and position of xmax in units of length
            * 'full' return trace, depth and charge_excess profile
        theta_reference: string (default: X0)
            * 'X0': viewing angle relativ to start of the shower
            * 'Xmax': viewing angle is relativ to Xmax, internally it will be converted to be relative to X0
            
        Returns: array of floats
            array of electric-field time trace in 'on-sky' coordinate system eR, eTheta, ePhi
        """
        if not shower_type in self._library.keys():
            raise KeyError("shower type {} not present in library. Available shower types are {}".format(shower_type, *self._library.keys()))
    
        # determine closes available energy in shower library
        energies = np.array(self._library[shower_type].keys())
        iE = np.argmin(np.abs(energies - shower_energy))
        rescaling_factor = shower_energy / energies[iE]
        logger.info("shower energy of {:.3g}eV requested, closest available energy is {:.3g}eV. The amplitude of the charge-excess profile will be rescaled accordingly by a factor of {:.2f}".format(shower_energy / units.eV, energies[iE] / units.eV, rescaling_factor))
        profiles = self._library[shower_type][energies[iE]]
        N_profiles = len(profiles['charge_excess'])
        
        if(iN is None):
            if(same_shower):
                if(shower_type in self._random_numbers):
                    iN = self._random_numbers[shower_type]
                    logger.info("using previously used shower {}/{}".format(iN, N_profiles))
                else:
                    logger.warning("no previous random number for shower type {} exists. Generating a new random number.".format(shower_type))
                    iN = np.random.randint(N_profiles)
                    self._random_numbers[shower_type] = iN
                    logger.info("picking profile {}/{} randomly".format(iN, N_profiles))
            else:
                iN = np.random.randint(N_profiles)
                self._random_numbers[shower_type] = iN
                logger.info("picking profile {}/{} randomly".format(iN, N_profiles))
        else:
            logger.info("using shower {}/{} as specified by user".format(iN, N_profiles))
            
        profile_depth = profiles['depth']
        profile_ce = profiles['charge_excess'][iN] * rescaling_factor
        
        xmax = profile_depth[np.argmax(profile_ce)]
        if(theta_reference == 'Xmax'):
            thetat = copy.copy(theta)
            theta = thetaprime_to_theta(theta, xmax, R)
            logger.info("transforming viewing angle from {:.2f} to {:.2f}".format(thetat / units.deg, theta / units.deg))
        elif(theta_reference != 'X0'):
            raise NotImplementedError("theta_reference = '{}' is not implemented".format(theta_reference))
        
        vp = get_vector_potential_fast(shower_energy, theta, N, dt, profile_depth, profile_ce, shower_type, n_index, R,
                                       self._interp_factor, self._interp_factor2, shift_for_xmax)
        trace = -np.diff(vp, axis=0) / dt
#         trace = -np.gradient(vp, axis=0) / dt
        
        # use viewing angle relative to shower maximum for rotation into spherical coordinate system (that reduced eR component)
        thetaprime = theta_to_thetaprime(theta, xmax, R)
        cs = cstrafo.cstrafo(zenith=thetaprime, azimuth=0)
        trace_onsky = cs.transform_from_ground_to_onsky(trace.T)
        if(output_mode == 'full'):
            return trace_onsky, profile_depth, profile_ce
        elif(output_mode == 'Xmax'):
            xmax = profile_depth[np.argmax(profile_ce)]
            Lmax = xmax / rho
            return trace_onsky, Lmax
        return trace_onsky
    
    
def get_vector_potential_fast(shower_energy, theta, N, dt, profile_depth, profile_ce,
                              shower_type="HAD", n_index=1.78, distance=1 * units.m,
                              interp_factor=1., interp_factor2=100., shift_for_xmax=False):
    """
    fast interpolation of time-domain calculation of vector potential of the 
    Askaryan pulse from a charge-excess profile
    
    Note that the returned array has N+1 samples so that the derivative (the efield) will have N samples. 
    
    The numerical integration was replaces by a sum using the trapeoiz rule using vectorized numpy operations
    
    Parameters
    ----------
    shower_energy: float
        the energy of the shower
    theta: float
        viewing angle, i.e., the angle between shower axis and launch angle of the signal (the ray path)
    N: int
        number of samples in the time domain
    dt: float
        size of one time bin in units of time
    profile_depth: array of floats
        shower depth values of the charge excess profile
    profile_ce: array of floats
        charge-excess values of the charge excess profile
    shower_type: string (default "HAD")
        type of shower, either "HAD" (hadronic), "EM" (electromagnetic) or "TAU" (tau lepton induced)
    n_index: float (default 1.78)
        index of refraction where the shower development takes place
    distance: float (default 1km)
        observation distance, the signal amplitude will be scaled according to 1/R
    interp_factor: int (default 1)
        interpolation factor of charge-excess profile. Results in a more precise numerical integration which might be beneficial 
        for small vertex distances but also slows down the calculation proportional to the interpolation factor.
        if None, the interpolation factor will be calculated from the distance 
    interp_factor2: int (default 100)
        interpolation just around the peak of the form factor 
    shift_for_xmax: bool (default True)
        if True the observer position is placed relative to the position of the shower maximum, if False it is placed 
        with respect to (0,0,0) which is the start of the charge-excess profile
    """
    
    ttt = np.arange(0, (N + 1) * dt, dt)
    ttt = ttt + 0.5 * dt - ttt.mean()
    if(len(ttt) != N + 1):
        ttt = ttt[:-1]
    N = len(ttt)

    xn = n_index
    cher = np.arccos(1. / n_index)
    beta = 1.

    profile_dense = profile_depth
    profile_ce_interp = profile_ce
    if(interp_factor != 1):
        profile_dense = np.linspace(min(profile_depth), max(profile_depth), interp_factor * len(profile_depth))
        profile_ce_interp = np.interp(profile_dense, profile_depth, profile_ce)
    length = profile_dense / rho
    dxmax = length[np.argmax(profile_ce_interp)]
#     theta2 = np.arctan(R * np.sin(theta)/(R * np.cos(theta) - dxmax))
#     logger.warning("theta changes from {:.2f} to {:.2f}".format(theta/units.deg, theta2/units.deg))
    
    # calculate antenna position in ARZ reference frame
    # coordinate system is with respect to an origin which is located
    # at the position where the primary particle is injected in the medium. The reference frame
    # is z = along shower axis, and x,y are two arbitray directions perpendicular to z
    # and perpendicular among themselves of course.
    # For instance to place an observer at a distance R and angle theta w.r.t. shower axis in the x,z plane
    # it can be simply done by putting in the input file the numerical values:
    X = np.array([distance * np.sin(theta), 0., distance * np.cos(theta)])
    if(shift_for_xmax):
        logger.info("shower maximum at z = {:.1f}m, shifting observer position accordingly.".format(dxmax / units.m))
        X = np.array([distance * np.sin(theta), 0., distance * np.cos(theta) + dxmax])
    logger.info("setting observer position to {}".format(X))

    def get_dist_shower(X, z):
        """
        Distance from position in shower depth z' to each antenna.
        Denominator in Eq. (22) PRD paper

        Parameters
        ----------
        X: 3dim np. array
            position of antenna in ARZ reference frame
        z: shower depth
        """
        return (X[0] ** 2 + X[1] ** 2 + (X[2] - z) ** 2) ** 0.5

    # calculate total charged track length
    xntot = np.sum(profile_ce_interp) * (length[1] - length[0])
    factor = -xmu / (4. * np.pi)
    fc = 4. * np.pi / (xmu * np.sin(cher))

    vp = np.zeros((N, 3))
    for it, t in enumerate(ttt):
        tobs = t + (get_dist_shower(X, 0) / c * xn)
        z = length

        R = get_dist_shower(X, z)
        arg = z - (beta * c * tobs - xn * R)
        
        # Note that Acher peaks at tt=0 which corresponds to the observer time.
        # The shift from tobs to tt=0 is done when defining argument
        tt = (-arg / (c * beta))  # Parameterisation of A_Cherenkov with t in ns
        
        mask = abs(tt) < 20. * units.ns
        if(np.sum(mask) == 0):  # 
            vp[it] = 0
            continue
        
        profile_dense2 = profile_dense
        profile_ce_interp2 = profile_ce_interp
        abc = False
        if(interp_factor2 != 1):
            # we only need to interpolate between +- 1ns to achieve a better precision in the numerical integration
            # the following code finds the indices sourrounding the bins fulfilling these condition
            # please not that we often have two distinct intervals having -1 < tt < 1
            tmask = (tt < 1 * units.ns) & (tt > -1 * units.ns)
            gaps = (tmask[1:] ^ tmask[:-1])  # xor
            indices = np.arange(len(gaps))[gaps]  # the indices in between tt is within -+ 1ns
            if(len(indices) != 0):  # only interpolate if we have time within +- 1 ns of the observer time
                # now we add the corner cases of having the tt array start or end with an entry fulfilling the condition
                if(len(indices) % 2 != 0):
                    if((tt[0] < 1 * units.ns) and (tt[0] > -1 * units.ns) and indices[0] != 0):
                        indices = np.append(0, indices)
                    else:
                        if(indices[-1] != (len(tt) - 1)):
                            indices = np.append(indices, len(tt) - 1)
                if(len(indices) % 2 == 0):  # this rejects the cases where only the first or the last entry fulfills the -1 < tt < 1 condition
                    dt = tt[1] - tt[0]
                    
                    dp = profile_dense2[1] - profile_dense2[0]
                    if(len(indices) == 2):  # we have only one interval
                        i_start = indices[0]
                        i_stop = indices[1]
                        profile_dense2 = np.arange(profile_dense[i_start], profile_dense[i_stop], dp / interp_factor2)
                        profile_ce_interp2 = np.interp(profile_dense2, profile_dense[i_start:i_stop], profile_ce_interp[i_start:i_stop])
                        profile_dense2 = np.append(np.append(profile_dense[:i_start], profile_dense2), profile_dense[i_stop:])
                        profile_ce_interp2 = np.append(np.append(profile_ce_interp[:i_start], profile_ce_interp2), profile_ce_interp[i_stop:])
                    elif(len(indices) == 4):  # we have two intervals, hence, we need to upsample two distinct intervals and put the full array back together. 
                        i_start = indices[0]
                        i_stop = indices[1]
                        profile_dense2 = np.arange(profile_dense[i_start], profile_dense[i_stop], dp / interp_factor2)
                        profile_ce_interp2 = np.interp(profile_dense2, profile_dense[i_start:i_stop], profile_ce_interp[i_start:i_stop])
                        
                        i_start3 = indices[2]
                        i_stop3 = indices[3]
                        profile_dense3 = np.arange(profile_dense[i_start3], profile_dense[i_stop3], dp / interp_factor2)
                        profile_ce_interp3 = np.interp(profile_dense3, profile_dense[i_start3:i_stop3], profile_ce_interp[i_start3:i_stop3])
                        
                        profile_dense2 = np.append(np.append(np.append(np.append(
                                                        profile_dense[:i_start], profile_dense2),
                                                           profile_dense[i_stop:i_start3]),
                                                              profile_dense3),
                                                                   profile_dense[i_stop3:])
                        profile_ce_interp2 = np.append(np.append(np.append(np.append(
                                                profile_ce_interp[:i_start],
                                                profile_ce_interp2),
                                                profile_ce_interp[i_stop:i_start3]),
                                                profile_ce_interp3),
                                                profile_ce_interp[i_stop3:])
                            
                    else:
                        raise NotImplementedError("length of indices is not 2 nor 4")  # this should never happen
                    if 0:
                        abc = True
                        i_stop = len(profile_dense) - 1
                        from matplotlib import pyplot as plt
                        fig, ax = plt.subplots(1, 1)
                        ax.plot(tt, color='0.5')
                        ax.plot(np.arange(len(tmask))[tmask], tt[tmask], 'o')
                        ax.plot(indices, np.ones_like(indices), 'd')
        #                 ax.plot(np.arange(len(tmask))[gaps], tt[gaps], 'd')
                        plt.show()
                
                    # recalculate parameters for interpolated values
                    z = profile_dense2 / rho
                    R = get_dist_shower(X, z)
                    arg = z - (beta * c * tobs - xn * R)
                    tt = (-arg / (c * beta))
                    mask = abs(tt) < 20. * units.ns
                    tmask = (tt < 1 * units.ns) & (tt > -1 * units.ns)
        
        F_p = np.zeros_like(tt)
        # Cut fit above +/-5 ns

        u_x = X[0] / R
        u_y = X[1] / R
        u_z = (X[2] - z) / R
        beta_z = 1.
        vperp_x = u_x * u_z * beta_z
        vperp_y = u_y * u_z * beta_z
        vperp_z = -(u_x * u_x + u_y * u_y) * beta_z
        v = np.array([vperp_x, vperp_y, vperp_z])
        """
        Function F_p Eq.(15) PRD paper.
        """
        # Factor accompanying the F_p in Eq.(15) in PRD paper
        beta = 1.
        if(np.sum(mask)):
            # Choose Acher between purely electromagnetic, purely hadronic or mixed shower
            # Eq.(16) PRD paper.
            # Refit of ZHAireS results => factor 0.88 in Af_e
            Af_e = -4.5e-14 * 0.88 * units.V * units.s
            Af_p = -3.2e-14 * units.V * units.s  # V s
            E_TeV = shower_energy / units.TeV
            Acher = np.zeros_like(tt)
            if(shower_type == "HAD"):
                mask2 = tt > 0 & mask
                if(np.sum(mask2)):
                    Acher[mask2] = Af_p * E_TeV * (np.exp(-np.abs(tt[mask2]) / (0.065 * units.ns)) + 
                                          (1. + 3.00 / units.ns * np.abs(tt[mask2])) ** (-2.65))  # hadronic
                mask2 = tt <= 0 & mask
                if(np.sum(mask2)):
                    Acher[mask2] = Af_p * E_TeV * (np.exp(-np.abs(tt[mask2]) / (0.043 * units.ns)) + 
                                          (1. + 2.92 / units.ns * np.abs(tt[mask2])) ** (-3.21))  # hadronic
            elif(shower_type == "EM"):
                mask2 = tt > 0 & mask
                if(np.sum(mask2)):
                    Acher[mask2] = Af_e * E_TeV * (np.exp(-np.abs(tt[mask2]) / (0.057 * units.ns)) + 
                                          (1. + 2.87 / units.ns * np.abs(tt[mask2])) ** (-3.00))  # electromagnetic
                mask2 = tt <= 0 & mask
                if(np.sum(mask2)):
                    Acher[mask2] = Af_e * E_TeV * (np.exp(-np.abs(tt[mask2]) / (0.030 * units.ns)) + 
                                          (1. + 3.05 / units.ns * np.abs(tt[mask2])) ** (-3.50))  # electromagnetic
            elif(shower_type == "TAU"):
                logger.error("Tau showers are not yet implemented")
                raise NotImplementedError("Tau showers are not yet implemented")
            else:
                msg = "showers of type {} are not implemented. Use 'HAD', 'EM' or 'TAU'".format(shower_type)
                logger.error(msg)
                raise NotImplementedError(msg)
            # Obtain "shape" of Lambda-function from vp at Cherenkov angle
            # xntot = LQ_tot in PRD paper
            F_p[mask] = Acher[mask] * fc / xntot
#         F_p[~mask] = 1.e-30 * fc / xntot
        F_p[~mask] = 0

        vp[it] = np.trapz(-v * profile_ce_interp2 * F_p / R, z)
        if  0:
            import matplotlib.pyplot as plt
            fig, ax = plt.subplots(1, 1)
            inte = -v * profile_ce_interp2 * F_p / R
            ax.plot(tt, inte[0], '-')
            ax.plot(tt, inte[1], '-')
            ax.plot(tt, inte[2], '-')
            ax.plot(tt[tmask], inte[0][tmask], 'o')
            ax.plot(tt[tmask], inte[1][tmask], 'o')
            ax.plot(tt[tmask], inte[2][tmask], 'o')
            ax.set_title("{}".format(vp[it]))
            plt.show()

    vp *= factor
    if 0:
        import matplotlib.pyplot as plt
        fig, (ax, ax2) = plt.subplots(1, 2)
        ax.plot(vp)
        print(vp.shape)
        t0 = -np.gradient(vp.T[0]) / dt
        t1 = -np.gradient(vp.T[1]) / dt
        t2 = -np.gradient(vp.T[2]) / dt
        trace2 = -np.diff(vp, axis=0) / dt
#         print(trace.shape)
        ax2.plot(t0)
        ax2.plot(t1)
        ax2.plot(t2)
        
        ax2.plot(trace2.T[0], '--')
        ax2.plot(trace2.T[1], '--')
        ax2.plot(trace2.T[2], '--')
        plt.show()
    return vp


def get_vector_potential(energy, theta, N, dt, y=1, ccnc='cc', flavor=12, n_index=1.78, R=1 * units.m,
                         profile_depth=None, profile_ce=None):
    """
    python transcription of original FORTRAN code
    """

    tt = np.arange(0, (N + 1) * dt, dt)
    tt = tt + 0.5 * dt - tt.mean()
#     tmin = tt.min()
#     tmax = tt.max()

#     tmin = -100 * units.ns
#     tmax = 100 * units.ns

#     tt = np.arange(tmin, tmax, dt)
#     tt += 0.5 * dt
    N = len(tt)

    xn = n_index
    cher = np.arccos(1. / n_index)
    beta = 1.

    # calculate antenna position in ARZ reference frame
    # coordinate system is with respect to an origin which is located
    # at the position where the primary particle is injected in the medium. The reference frame
    # is z = along shower axis, and x,y are two arbitray directions perpendicular to z
    # and perpendicular among themselves of course.
    # For instance to place an observer at a distance R and angle theta w.r.t. shower axis in the x,z plane
    # it can be simply done by putting in the input file the numerical values:
    X = np.array([R * np.sin(theta), 0., R * np.cos(theta)])

    def get_dist_shower(X, z):
        """
        Distance from position in shower depth z' to each antenna.
        Denominator in Eq. (22) PRD paper

        Parameters
        ----------
        X: 3dim np. array
            position of antenna in ARZ reference frame
        z: shower depth
        """
        return (X[0] ** 2 + X[1] ** 2 + (X[2] - z) ** 2) ** 0.5

    length = profile_depth / rho
    xnep = intp.interp1d(length, profile_ce, bounds_error=False, fill_value=0)

    # calculate total charged track length
    xntot = np.sum(profile_ce) * (length[1] - length[0])
    # print("{:.5g}".format(xntot))
    # res = int.quad(xnep, length.min(), length.max())
    # print("{:.5g} {:.5g}".format(*res))

    if 0:  # debug plot
        ll = np.linspace(length.min(), length.max(), 10000)
        plt.plot(ll, xnep(ll))
        plt.plot(length, N_e - N_p, 'o')
        plt.show()

    factor = -xmu / (4. * np.pi)

    def xintegrand(z, index, tobs):
        R = get_dist_shower(X, z)
        arg = z - (beta * c * tobs - xn * R)
        u_x = X[0] / R
        u_y = X[1] / R
        u_z = (X[2] - z) / R
        beta_z = 1.
        vperp_x = u_x * u_z * beta_z
        vperp_y = u_y * u_z * beta_z
        vperp_z = -(u_x * u_x + u_y * u_y) * beta_z
        v = np.array([vperp_x, vperp_y, vperp_z])[index]

        return -v * xnep(z) * F_p(arg) / R

    def F_p(arg):
        """
        Function F_p Eq.(15) PRD paper.
        """
        # Factor accompanying the F_p in Eq.(15) in PRD paper
        fc = 4. * np.pi / (xmu * np.sin(cher))
        beta = 1.

        # Note that Acher peaks at tt=0 which corresponds to the observer time.
        # The shift from tobs to tt=0 is done when defining argument
        tt = (-arg / (c * beta))  # Parameterisation of A_Cherenkov with t in ns
        # Cut fit above +/-5 ns
        if (abs(tt) > 5. * units.ns):
            return 1.e-30 * fc / xntot

        # Choose Acher between purely electromagnetic, purely hadronic or mixed shower
        # Eq.(16) PRD paper.
        # Refit of ZHAireS results => factor 0.88 in Af_e
        Af_e = -4.5e-14 * 0.88 * units.V * units.s
        Af_p = -3.2e-14 * units.V * units.s  # V s
        E_TeV = energy / units.TeV
        if (tt > 0):
            A_e = Af_e * E_TeV * (np.exp(-np.abs(tt) / (0.057 * units.ns)) + 
                                  (1. + 2.87 / units.ns * np.abs(tt)) ** (-3.00))  # electromagnetic
            A_p = Af_p * E_TeV * (np.exp(-np.abs(tt) / (0.065 * units.ns)) + 
                                  (1. + 3.00 / units.ns * np.abs(tt)) ** (-2.65))  # hadronic
        else:
            A_e = Af_e * E_TeV * (np.exp(-np.abs(tt) / (0.030 * units.ns)) + 
                                  (1. + 3.05 / units.ns * np.abs(tt)) ** (-3.50))  # electromagnetic
            A_p = Af_p * E_TeV * (np.exp(-np.abs(tt) / (0.043 * units.ns)) + 
                                  (1. + 2.92 / units.ns * np.abs(tt)) ** (-3.21))  # hadronic

        if(ccnc == 'nc'):
            Acher = y * A_p
        else:
            if(np.abs(flavor) == 12):
                Acher = (1. - y) * A_e + y * A_p
            else:
                Acher = 0

        # Obtain "shape" of Lambda-function from vp at Cherenkov angle
        # xntot = LQ_tot in PRD paper
        return Acher * fc / xntot

    vp = np.zeros((N, 3))
    for it, t in enumerate(tt):
        tobs = t + (get_dist_shower(X, 0) / c * xn)
        xmin = length.min()
        xmax = length.max()
        if(X[0] != 0):
            vp[it][0] = int.quad(xintegrand, xmin, xmax, args=(0, tobs))[0]
        if(X[1] != 0):
            vp[it][1] = int.quad(xintegrand, xmin, xmax, args=(1, tobs))[0]
        if(X[2] != 0):
            vp[it][2] = int.quad(xintegrand, xmin, xmax, args=(2, tobs))[0]
    vp *= factor
    return vp


class ARZ_tabulated(object):
    __instance = None

    def __new__(cls, seed=1234, library=None):
        if ARZ_tabulated.__instance is None:
            ARZ_tabulated.__instance = object.__new__(cls, seed, library)
        return ARZ_tabulated.__instance

    def __init__(self, seed=1234, library=None):
        logger.warning("setting seed to {}".format(seed))
        np.random.seed(seed)
        self._random_numbers = {}
        self._version = (1, 1)
        # # load shower library into memory
        if(library is None):
            library = os.path.join(os.path.dirname(__file__), "shower_library/ARZ_library_v{:d}.{:d}.pkl".format(*self._version))
        else:
            if(not os.path.exists(library)):
                logger.error("user specified pulse library {} not found.".format(library))
                raise FileNotFoundError("user specified pulse library {} not found.".format(library))
        self.__check_and_get_library()
        
        with open(library) as fin:
            logger.warning("loading pulse library into memory")
            self._library = pickle.load(fin)
            
    def __check_and_get_library(self):
        """
        checks if pulse library exists and is up to date by comparing the sha1sum. If the library does not exist
        or changes on the server, a new library will be downloaded. 
        """
        path = os.path.join(os.path.dirname(__file__), "shower_library/ARZ_library_v{:d}.{:d}.pkl".format(*self._version))
        
        download_file = False
        if(not os.path.exists(path)):
            logger.warning("ARZ library version {} does not exist on the local file system yet. It will be downloaded to {}".format(self._version, path))
            download_file = True
    
        if(os.path.exists(path)):
            BUF_SIZE = 65536 * 2 ** 4  # lets read stuff in 64kb chunks!
            import hashlib
            import json
            sha1 = hashlib.sha1()
            with open(path, 'rb') as f:
                while True:
                    data = f.read(BUF_SIZE)
                    if not data:
                        break
                    sha1.update(data)
    
            shower_directory = os.path.join(os.path.dirname(__file__), "shower_library/")
            with open(os.path.join(shower_directory, 'shower_lib_hash.json'), 'r') as fin:
                lib_hashs = json.load(fin)
                if("ARZ_{:d}.{:d}".format(*self._version) in lib_hashs.keys()):
                    if(sha1.hexdigest() != lib_hashs["{:d}.{:d}".format(*self._version)]):
                        logger.warning("pulse library {} has changed on the server. downloading newest version...".format(self._version))
                        download_file = True
                else:
                    logger.warning("no hash sum of {} available, skipping up-to-date check".format(os.path.basename(path)))            
        if not download_file:
            return True
        else:
            import requests
            URL = 'http://arianna.ps.uci.edu/~arianna/data/ce_shower_library/ARZ_library_v{:d}.{:d}.pkl'.format(*self._version)
    
            logger.info("downloading pulse library {} from {}. This can take a while...".format(self._version, URL))
            r = requests.get(URL)
            if (r.status_code != requests.codes.ok):
                logger.error("error in download of antenna model")
                raise IOError("error in download of antenna model")
            with open(path, "wb") as code:
                code.write(r.content)
            logger.info("...download finished.")
        
    def set_seed(self, seed):
        """
        allow to set a new random seed
        """
        np.random.seed(seed)
        
    def get_time_trace(self, shower_energy, theta, N, dt, shower_type, n_index, R,
                       same_shower=False, iN=None, output_mode='trace', theta_reference='X0'):
        """
        calculates the electric-field Askaryan pulse from a charge-excess profile
        
        Parameters
        ----------
        shower_energy: float
            the energy of the shower
        theta: float
            viewing angle, i.e., the angle between shower axis and launch angle of the signal (the ray path)
        N: int
            number of samples in the time domain
        dt: float
            size of one time bin in units of time
        shower_type: string (default "HAD")
            type of shower, either "HAD" (hadronic), "EM" (electromagnetic) or "TAU" (tau lepton induced)
        n_index: float (default 1.78)
            index of refraction where the shower development takes place
        R: float (default 1km)
            observation distance, the signal amplitude will be scaled according to 1/R
        same_shower: bool (default False)
            if False, for each request a new random shower realization is choosen. 
            if True, the shower from the last request of the same shower type is used. This is needed to get the Askaryan
            signal for both ray tracing solutions from the same shower. 
        iN: int or None (default None)
            specify shower number
        output_mode: string
            * 'trace' (default): return only the electric field trace
            * 'Xmax': return trace and position of xmax in units of length
        theta_reference: string (default: X0)
            * 'X0': viewing angle relativ to start of the shower
            * 'Xmax': viewing angle is relativ to Xmax, internally it will be converted to be relative to X0
            
        Returns: array of floats
            array of electric-field time trace in 'on-sky' coordinate system eR, eTheta, ePhi
        """
        if not shower_type in self._library.keys():
            raise KeyError("shower type {} not present in library. Available shower types are {}".format(shower_type, *self._library.keys()))
    
        # determine closes available energy in shower library
        energies = np.array(list(self._library[shower_type].keys()))
        iE = np.argmin(np.abs(energies - shower_energy))
        rescaling_factor = shower_energy / energies[iE]
        logger.info("shower energy of {:.3g}eV requested, closest available energy is {:.3g}eV. The pulse amplitude will be rescaled accordingly by a factor of {:.2f}".format(shower_energy / units.eV, energies[iE] / units.eV, rescaling_factor))
        profiles = self._library[shower_type][energies[iE]]
        N_profiles = len(profiles.keys())
        
        if(iN is None):
            if(same_shower):
                if(shower_type in self._random_numbers):
                    iN = self._random_numbers[shower_type]
                    logger.info("using previously used shower {}/{}".format(iN, N_profiles))
                else:
                    logger.warning("no previous random number for shower type {} exists. Generating a new random number.".format(shower_type))
                    iN = np.random.randint(N_profiles)
                    self._random_numbers[shower_type] = iN
                    logger.info("picking profile {}/{} randomly".format(iN, N_profiles))
            else:
                iN = np.random.randint(N_profiles)
                self._random_numbers[shower_type] = iN
                logger.info("picking profile {}/{} randomly".format(iN, N_profiles))
        else:
            logger.info("using shower {}/{} as specified by user".format(iN, N_profiles))
            
        thetas = profiles[iN].keys()
        iT = np.argmin(np.abs(thetas - theta))
        logger.info("selecting theta = {:.2f} ({:.2f} requested)".format(thetas[iT] / units.deg, theta))
        trace = profiles[iT]['trace']
        t0 = profiles[iT]['t0']
        Lmax = profiles[iT]['Lmax']
        trace2 = np.zeros(N)
        tcenter = N // 2 * dt
        tstart = t0 + tcenter
        i0 = np.int(np.round(tstart / dt))
        trace2[i0:(i0 + len(trace))] = trace
        
        trace2 *= self._library['meta']['R'] / R * rescaling_factor
        
        if(output_mode == 'Xmax'):
            return trace2, Lmax
        return trace2


if __name__ == "__main__":
    energy = 1.e6 * units.TeV
    theta = 55 * units.deg
    R = 1 * units.km
    N = 512
    dt = 0.5 * units.ns
    n_index = 1.78
    y = 0.60146725
    ccnc = 'cc'
    flavor = 12  # e = 12, mu = 14, tau = 16

    cdir = os.path.dirname(__file__)
    bins, depth_e, N_e = np.loadtxt(os.path.join(cdir, "shower_library/nue_1EeV_CC_1_s0001.t1005"), unpack=True)
    bins, depth_p, N_p = np.loadtxt(os.path.join(cdir, "shower_library/nue_1EeV_CC_1_s0001.t1006"), unpack=True)
    depth_e *= units.g / units.cm ** 2
    depth_p *= units.g / units.cm ** 2
    depth_e -= 1000 * units.g / units.cm ** 2  # all simulations have an artificial offset of 1000 g/cm^2
    depth_p -= 1000 * units.g / units.cm ** 2
    # sanity check if files electron and positron profiles are compatible
    if (not np.all(depth_e == depth_p)):
        raise ImportError("electron and positron profile have different depths")

    vp = get_vector_potential(energy, theta, N, dt, y, ccnc, flavor, n_index, R, profile_depth=depth_e, profile_ce=(N_e - N_p))
    vp2 = get_vector_potential(energy, theta, N, dt, y, "EM", n_index, R, profile_depth=depth_e, profile_ce=(N_e - N_p))

    # generate time array
    tt = np.arange(0, (N + 1) * dt, dt)
    tt = tt + 0.5 * dt - tt.mean()

    t, Ax, Ay, Az = np.loadtxt("fortran_reference.dat", unpack=True)
    fig, ax = plt.subplots(1, 1)
    ax.plot(tt, vp[:, 0] / units.V / units.s)
    ax.plot(tt, vp[:, 1] / units.V / units.s)
    ax.plot(tt, vp[:, 2] / units.V / units.s)
    ax.plot(t, Ax, "C0--")
    ax.plot(t, Az, "C2--")
    ax.set_xlim(-2, 2)

    ax.set_xlabel("time [ns]")
    ax.set_ylabel("vector potential")

    mask = np.array([x in t for x in tt])
    fig, ax = plt.subplots(1, 1)
    ax.plot(t, vp[:, 0][mask] / units.V / units.s / Ax)
    ax.plot(t, vp[:, 2][mask] / units.V / units.s / Az)
    ax.set_xlim(-2, 2)
    ax.set_xlabel("time [ns]")
    ax.set_ylabel("python/fortran implementation")
    ax.set_ylim(0.8, 1.2)

    trace = get_time_trace(energy, theta, N, dt, y, ccnc, flavor, n_index, R)
    tt = np.arange(0, dt * N, dt)
    fig, ax = plt.subplots(1, 1)
    ax.plot(tt, trace[:, 0])
    ax.plot(tt, trace[:, 1])
    ax.plot(tt, trace[:, 2])
    fig.tight_layout()
    plt.show()



"""
HDF5 files tools for HDF5 Manipulator
"""

import h5py


def load(filename):

    """Load hdf5 file to data dictionary and return it.

    Keyword arguments:
    filename -- the full path to hdf5 file
    """

    f = h5py.File(filename, 'r')

    data = {}
    attrs = {}
    groups = {}
    group_attrs = {}

    for key in f:
        if isinstance(f[key], h5py._hl.group.Group):
            groups[key] = {}
            group_attrs[key] = {}
            for key2 in f[key]:
                groups[key][key2] = f[key][key2][...]
            for key2 in f[key].attrs:
                group_attrs[key][key2] = f[key].attrs[key2]
            
        else:
            data[key] = f[key][...]
        
    for key in f.attrs:
        attrs[key] = f.attrs[key]


    f.close()

    return data, attrs, groups, group_attrs


def save(filename, data, attrs, groups, group_attrs):

    """Create hdf5 file with given data.

    Keyword arguments:
    filename -- the full path to hdf5 file
    data -- dictionary with data
    """

    f = h5py.File(filename, 'w')

    for key in data:
        f.create_dataset(key, data[key].shape, dtype=data[key].dtype,
                         compression='gzip')[...] = data[key]
    
    for key in groups:
        g = f.create_group(key)
        for key2 in groups[key]:
            g[key2] = groups[key][key2]
        for key2 in group_attrs[key]:
            g.attrs[key2] = group_attrs[key][key2]
    
    for key in attrs:
        f.attrs[key] = attrs[key]
    
    f.close()


def save_subset(filename, data, begin, end):

    """Create hdf5 file with subset [begin, end) of given data.

    Keyword arguments:
    filename -- the full path to hdf5 file
    data -- dictionary with data
    begin -- start saving from index=i
    end -- finish savin at index=end
    """

    subset = {}

    for key in data:
        subset[key] = data[key][begin:end]

    save(filename, subset)


def save_subset_big(filename, data, begin, end):

    """Create hdf5 file with subset [begin, end) of given data.

    Keyword arguments:
    filename -- the full path to hdf5 file
    data -- input file
    begin -- start saving from index=i
    end -- finish savin at index=end
    """

    o = h5py.File(filename, 'w')

    for key in data:
        shape = list(data[key].shape)
        shape[0] = end - begin
        o.create_dataset(key, shape, dtype=data[key].dtype,
                         compression='gzip')[...] = data[key][begin:end]

    o.close()



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function
import numpy as np
from NuRadioMC.utilities import units, fft
from scipy import constants
import logging
logger = logging.getLogger("SignalGen.parametrizations")


def set_log_level(level):
    logger.setLevel(level)


"""

Analytic parametrizations of the radio pulse produced by an in-ice particle shower.

Generic functions to provide the frequency spectrum and the pulse in the time domain
are defined. All models/parametrizations should be added to each of these functions,
such that different parametrizations can be exchanged by just modifying the 'model'
argument of the respective function.

"""


def get_parametrizations():
    """ returns a list of all implemented parametrizations """
    return ['ZHS1992', 'Alvarez2000', 'Alvarez2009', 'Alvarez2012']


def get_time_trace(energy, theta, N, dt, shower_type, n_index, R, model):
    """
    returns the Askaryan pulse in the time domain of the eTheta component

    We implement only the time-domain solution and obtain the frequency spectrum
    via FFT (with the standard normalization of NuRadioMC). This approach assures
    that the units are interpreted correctly. In the time domain, the amplitudes
    are well defined and not details about fourier transform normalizations needs
    to be known by the user.

    Parameters
    ----------
    energy : float
        energy of the shower
    theta: float
        viewangle: angle between shower axis (neutrino direction) and the line
        of sight between interaction and detector
    N : int
        number of samples in the time domain
    dt: float
        time bin width, i.e. the inverse of the sampling rate
    shower_type: string (default "HAD")
        type of shower, either "HAD" (hadronic), "EM" (electromagnetic)
    n_index: float
        index of refraction at interaction vertex
    R: float
        distance from vertex to observer
    model: string
        specifies the signal model
        * ZHS1992: the original ZHS parametrization from E. Zas, F. Halzen, and T. Stanev, Phys. Rev. D 45, 362 (1992), doi:10.1103/PhysRevD.45.362, this parametrization does not contain any phase information
        * Alvarez2000: parameterization based on ZHS mainly based on J. Alvarez-Muniz, R. A. Vazquez, and E. Zas, Calculation methods for radio pulses from high energyshowers,Physical Review D62 (2000) https://doi.org/10.1103/PhysRevD.84.103003
        * Alvarez2009: parameterization based on ZHS from J. Alvarez-Muniz, W. R. Carvalho, M. Tueros, and E. Zas, Coherent cherenkov radio pulses fromhadronic showers up to eev energies,Astroparticle Physics35(2012), no. 6 287 – 299 and J. Alvarez-Muniz, C. James, R. Protheroe, and E. Zas, Thinned simulations of extremely energeticshowers in dense media for radio applications, Astroparticle Physics 32 (2009), no. 2 100 – 111

    Returns
    -------
    spectrum: array
        the complex amplitudes for the given frequencies

    """
    if(model == 'ZHS1992'):
        """ Parametrization from E. Zas, F. Halzen, and T. Stanev, Phys. Rev. D 45, 362 (1992)."""
        freqs = np.fft.rfftfreq(N, dt)
        vv0 = freqs / (0.5 * units.GHz)
        cherenkov_angle = np.arccos(1. / n_index)
        domega = (theta - cherenkov_angle)
        tmp = np.exp(+0.5j * np.pi)  # set phases to 90deg
        tmp *= 1.1e-7 * energy / units.TeV * vv0 * 1. / \
            (1 + 0.4 * (vv0) ** 2) * np.exp(-0.5 * (domega / (2.4 * units.deg / vv0)) ** 2) * \
            units.V / units.m / (R / units.m) / units.MHz
        # the factor 0.5 is introduced to compensate the unusual fourier transform normalization used in the ZHS code
        trace = 0.5 * np.fft.irfft(tmp) / dt
        trace = np.roll(trace, int(2 * units.ns / dt))
        return trace

    elif(model == 'Alvarez2009'):

        # This parameterisation is not very accurate for energies above 10 EeV
        # The ARZ model should be used instead
        freqs = np.fft.rfftfreq(N, dt)[1:] # exclude zero frequency

        E_C = 73.1 * units.MeV
        rho = 0.924 * units.g / units.cm**3
        X_0 = 36.08 * units.g / units.cm**2
        R_M = 10.57 * units.g / units.cm**2
        c = constants.c * units.m / units.s

        def A(E_0, theta, freq):

            if (shower_type == 'HAD'):
                k_E_0 = 4.13e-16 * units.V / units.cm / units.MHz**2
                k_E_1 = 2.54
                log10_E_E = 10.60
                k_E_bar = k_E_0 * np.tanh( (np.log10(E_0/units.eV)-log10_E_E)/k_E_1 )
            elif (shower_type == 'EM'):
                k_E_bar = 4.65e-16 * units.V / units.cm / units.MHz**2

            return k_E_bar * E_0/E_C * X_0/rho * np.sin(theta) * freq

        def nu_L(E_0, theta):

            if (shower_type == 'HAD'):
                k_L_0 = 31.25
                gamma = 3.01e-2
                E_L = 1.e15 * units.eV
                k_L = k_L_0 * (E_0/E_L)**gamma
            elif (shower_type == 'EM'):
                sigma_0 = 3.39e-2
                log10_E_sigma = 14.99
                delta_0 = 0
                delta_1 = 2.25e-2
                log10_E_0 = np.log10(E_0/units.eV)
                if ( log10_E_0 < log10_E_sigma ):
                    sigma_k_L = sigma_0 + delta_0 * (log10_E_0-log10_E_sigma)
                else:
                    sigma_k_L = sigma_0 + delta_1 * (log10_E_0-log10_E_sigma)

                log10_k_0 = 1.52
                log10_E_LPM = 16.61
                gamma_0 = 5.59e-2
                gamma_1 = 0.39
                if ( log10_E_0 < log10_E_LPM ):
                    log10_k_L_bar = log10_k_0 + gamma_0 * ( log10_E_0 - log10_E_LPM )
                else:
                    log10_k_L_bar = log10_k_0 + gamma_1 * ( log10_E_0 - log10_E_LPM )

                k_L = 10**np.random.normal(log10_k_L_bar, sigma_k_L)

            nu_L = rho/k_L/X_0

            cher_cut = 1.e-8
            if ( np.abs(1-n_index*np.cos(theta)) < cher_cut):
                nu_L *= c/cher_cut
            else:
                nu_L *= c/np.abs(1-n_index*np.cos(theta))

            return nu_L

        def d_L(E_0, theta, freq):

            if (shower_type == "HAD"):
                beta = 2.57
            elif (shower_type == "EM"):
                beta = 2.74

            return 1/(1+(freq/nu_L(E_0, theta))**beta)

        def d_R(E_0, theta, freq):

            if (shower_type == "HAD"):
                k_R_0 = 2.73
                k_R_1 = 1.72
                log10_E_R = 12.92
                k_R_bar = k_R_0 + np.tanh( (log10_E_R-np.log10(E_0/units.eV))/k_R_1 )
            elif (shower_type == "EM"):
                k_R_bar = 1.54
            nu_R = rho/k_R_bar/R_M * c/np.sqrt(n_index**2-1)

            alpha = 1.27
            return 1/(1+(freq/nu_R)**alpha)

        spectrum = A(energy, theta, freqs) * d_L(energy, theta, freqs) * d_R(energy, theta, freqs)
        spectrum *= 0.5 # ZHS Fourier transform normalisation
        spectrum /= R
        spectrum = np.insert(spectrum, 0, 0)

        trace = np.fft.irfft(spectrum * np.exp(0.5j * np.pi)) / dt  # set phases to 90deg
        trace = np.roll(trace, int(50 * units.ns / dt))
        return trace

    elif(model == 'Alvarez2000'):
        freqs = np.fft.rfftfreq(N, dt)[1:]  # exclude zero frequency
        cherenkov_angle = np.arccos(1. / n_index)

        Elpm = 2e15 * units.eV
        dThetaEM = 2.7 * units.deg * 500 * units.MHz / freqs * (Elpm / (0.14 * energy + Elpm)) ** 0.3
#         logger.debug("dThetaEM = {}".format(dThetaEM))

        epsilon = np.log10(energy / units.TeV)
        dThetaHad = 0
        if (epsilon >= 0 and epsilon <= 2):
            dThetaHad = 500 * units.MHz / freqs * (2.07 - 0.33 * epsilon + 7.5e-2 * epsilon ** 2) * units.deg
        elif (epsilon > 2 and epsilon <= 5):
            dThetaHad = 500 * units.MHz / freqs * (1.74 - 1.21e-2 * epsilon) * units.deg
        elif(epsilon > 5 and epsilon <= 7):
            dThetaHad = 500 * units.MHz / freqs * (4.23 - 0.785 * epsilon + 5.5e-2 * epsilon ** 2) * units.deg
        elif(epsilon > 7):
            dThetaHad = 500 * units.MHz / freqs * (4.23 - 0.785 * 7 + 5.5e-2 * 7 ** 2) * \
                (1 + (epsilon - 7) * 0.075) * units.deg

        f0 = 1.15 * units.GHz
        E = 2.53e-7 * energy / units.TeV * freqs / f0 / (1 + (freqs / f0) ** 1.44)
        E *= units.V / units.m / units.MHz
        E *= np.sin(theta) / np.sin(cherenkov_angle)

        tmp = np.zeros(len(freqs) + 1)
        if(shower_type == "EM"):
            tmp[1:] = E * np.exp(-np.log(2) * ((theta - cherenkov_angle) / dThetaEM) ** 2) / R
        elif(shower_type == "HAD"):
            if(np.any(dThetaHad != 0)):
                tmp[1:] = E * np.exp(-np.log(2) * ((theta - cherenkov_angle) / dThetaHad) ** 2) / R

                def missing_energy_factor(E_0):
                    # Missing energy factor for hadronic cascades
                    # Taken from DOI: 10.1016/S0370-2693(98)00905-8
                    epsilon = np.log10(E_0/units.TeV)
                    f_epsilon  = -1.27e-2 - 4.76e-2*(epsilon+3)
                    f_epsilon += -2.07e-3*(epsilon+3)**2 + 0.52*np.sqrt(epsilon+3)
                    return f_epsilon

                tmp[1:] *= missing_energy_factor(energy)
            else:
                pass
                # energy is below a TeV, setting Askaryan pulse to zero
        else:
            raise NotImplementedError("shower type {} not implemented in {} Askaryan module".format(shower_type, model))

        tmp *= 0.5  # the factor 0.5 is introduced to compensate the unusual fourier transform normalization used in the ZHS code

#         df = np.mean(freqs[1:] - freqs[:-1])
        trace = np.fft.irfft(tmp * np.exp(0.5j * np.pi)) / dt  # set phases to 90deg
        trace = np.roll(trace, int(50 * units.ns / dt))
        return trace

    else:
        raise NotImplementedError("model {} unknown".format(model))



import matplotlib.pyplot as plt
import numpy as np
import time
from NuRadioMC.SignalProp import analyticraytraycing as ray
from NuRadioMC.utilities import units, medium
import logging
import time
from radiotools import helper as hp
from radiotools import plthelpers as php
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('test_raytracing')

ice = medium.southpole_simple()

np.random.seed(0)  # set seed to have reproducible results
n_events = int(1e3)
rmin = 50. * units.m
rmax = 3. * units.km
zmin = 0. * units.m
zmax = -3. * units.km
rr = np.random.triangular(rmin, rmax, rmax, n_events)
phiphi = np.random.uniform(0, 2 * np.pi, n_events)
xx = rr * np.cos(phiphi)
yy = rr * np.sin(phiphi)
zz = np.random.uniform(zmin, zmax, n_events)

points = np.array([xx, yy, zz]).T
x_receiver = np.array([0., 0., -5.])

results_C0s_cpp = np.zeros((n_events, 2))
n_freqs = 256/2 + 1
# n_freqs = 5
results_A_cpp = np.zeros((n_events, 2, n_freqs))
t_start = time.time()
ff = np.linspace(0, 500*units.MHz, n_freqs)
# tt = 0
for iX, x in enumerate(points):
#     t_start2 = time.time()
    r = ray.ray_tracing(x, x_receiver, ice)
#     tt += (time.time() - t_start2)
    r.find_solutions()
    if(r.has_solution()):
        for iS in range(r.get_number_of_solutions()):
            results_C0s_cpp[iX, iS] = r.get_results()[iS]['C0']
            results_A_cpp[iX, iS] = r.get_attenuation(iS, ff)
t_cpp = time.time() - t_start
print("CPP time = {:.1f} seconds = {:.2f}ms/event".format(t_cpp, 1000. * t_cpp / n_events))
# print("CPP time = {:.1f} seconds = {:.2f}ms/event".format(tt, 1000. * tt / n_events))


results_C0s_python = np.zeros((n_events, 2))
results_A_python = np.zeros((n_events, 2, n_freqs))
ray.cpp_available = False
t_start = time.time()
for iX, x in enumerate(points):
    r = ray.ray_tracing(x, x_receiver, ice)
    r.find_solutions()
    if(r.has_solution()):
        for iS in range(r.get_number_of_solutions()):
            results_C0s_python[iX, iS] = r.get_results()[iS]['C0']
            results_A_python[iX, iS] = r.get_attenuation(iS, ff)
t_python = time.time() - t_start
print("Python time = {:.1f} seconds = {:.2f}ms/event".format(t_python, 1000. * t_python / n_events))

print("consistent results for C0: {}".format(np.allclose(results_C0s_cpp, results_C0s_python)))
print("consistent results for attenuation length: {}".format(np.allclose(results_A_cpp, results_A_python)))
print("consistent results for attenuation length rtol=1e-2, atol=1e-3: {}".format(np.allclose(results_A_cpp, results_A_python, rtol=1e-2, atol=1e-3)))



from __future__ import print_function, division
import numpy as np
from NuRadioMC.utilities import units
import os
from scipy import interpolate as intp
import glob
import pickle
import sys
from radiotools import plthelpers as php
from matplotlib import pyplot as plt
from NuRadioMC.SignalGen.ARZ import ARZ
import logging
from scipy import signal
import matplotlib.gridspec as gridspec

logger = logging.getLogger("test")
logging.basicConfig(level=logging.WARNING)
rho = 0.924 * units.g / units.cm ** 3  # density g cm^-3

plot=False

if __name__ == "__main__":
    sampling_rate = 5 * units.GHz
    dt = 1. / sampling_rate
    T2 = 50 * units.ns
    N2 = np.int(np.round(T2 / dt))
    n_index = 1.78
    theta = np.arccos(1. / n_index)
    R = 5 * units.km
    T = 600 * units.ns
    N = np.int(np.round(T / dt))
    tt = np.arange(0, dt * N, dt)
    
    dCs = np.append(np.arange(-20, -5, 1), np.append(np.arange(-5,-1,0.2), np.arange(-1, 0, .05)))
    dCs = np.append(np.append(dCs, [0]), -1 * dCs)
    dCs = np.sort(dCs) * units.deg
    print('generating Askaryan pulses for the following viewing angles')
    print(dCs/units.deg)
    
    lib_path = "/Users/cglaser/work/ARIANNA/data/ARZ/v1.1/library_v1.1.pkl"
    a = ARZ.ARZ(library=lib_path, interp_factor=100)
    
    showers = {}
    showers['meta'] = {'dt': dt,
                       'n_index': n_index,
                       'R': R}
    
    with open(lib_path, 'rb') as fin:
        lib = pickle.load(fin)
        for iS, shower_type in enumerate(lib):  # loop through shower types
            if(shower_type not in showers):
                showers[shower_type] = {}
            print("shower type {}".format(shower_type))
            b = []
            nb = []
            for iE, E in enumerate(lib[shower_type]):  # loop through energies
                print('E = {:.2g}eV'.format(E))
                
                nE = len(lib[shower_type][E]['charge_excess'])
                if(E not in showers[shower_type]):
                    showers[shower_type][E] = {}
                for iN in range(nE):  # loop through shower realizations
                    if(iN not in showers[shower_type][E]):
                        showers[shower_type][E][iN] = {}
                    if plot:
                        fig, ax = plt.subplots(1, 1)
                        fig2, ax2 = plt.subplots(1, 1)
                    for dC in dCs:  # loop through different cherenkov angles
                        theta1 = theta + dC
                        trace, Lmax = a.get_time_trace(E, theta1, N, dt, shower_type, n_index, R, iN=iN, output_mode='Xmax')
                        iMax = np.argmax(np.abs(trace[1]))
                        i1 = iMax - N2 // 2
                        if((i1 + N2) >= N) or (i1 < 0):
                            print("tracelength to short, i1 = {}".format(i1))
#                             raise IndexError("tracelength to short, i1 = {}".format(i1))
                        t0 = tt[i1] - tt.mean()
                        showers[shower_type][E][iN][theta1] = {}
                        showers[shower_type][E][iN][theta1]['t0'] = t0
                        showers[shower_type][E][iN][theta1]['Lmax'] = Lmax
                        showers[shower_type][E][iN][theta1]['trace'] = trace[1][i1:(i1 + N2)]
                        if(plot):
                            ax.plot(tt[i1:(i1 + N2)], trace[1][i1:(i1 + N2)] / np.abs(trace[1]).max())
                            ax2.plot(tt[i1:(i1 + N2)], np.abs(signal.hilbert(trace[1][i1:(i1 + N2)])))
                    if(plot):
                        ax.set_title("{} E = {:.2g}eV i = {:d}".format(shower_type, E / units.eV, iN))
                        ax2.set_title("{} E = {:.2g}eV i = {:d}".format(shower_type, E / units.eV, iN))
                        ax.set_xlabel('time [ns]')
                        ax2.set_xlabel('time [ns]')
                        ax.set_ylabel('normalized amplitude')
                        ax2.semilogy(True)
                        fig.tight_layout()
                        fig2.tight_layout()
#                         plt.show()
                        fig.savefig("plots/{}_{:.2g}eV_{:03d}.png".format(shower_type, E / units.eV, iN))
                        fig2.savefig("plots/{}_{:.2g}eV_{:03d}_log.png".format(shower_type, E / units.eV, iN))
                        plt.close("all")
                            
        with open("ARZ_library_v1.1.pkl", 'wb') as fout:
            pickle.dump(showers, fout, protocol=2)



from __future__ import absolute_import, division, print_function
import numpy as np
from NuRadioMC.utilities import units
import argparse
import h5py

parser = argparse.ArgumentParser(description='Run NuRadioMC simulation')
parser.add_argument('hdf5input', type=str,
                    help='path to NuRadioMC hdf5 input event list')
parser.add_argument('AraSiminput', type=str,
                    help='path to AraSim input file')
parser.add_argument('avgDepth', type=float,
                    help='average depth of all antennas in meters (use positive numbers)')
args = parser.parse_args()

fin = h5py.File(args.hdf5input, 'r')

#AraStationDep = 178.5598 / units.m
AraStationDep = args.avgDepth / units.m
event_ids = fin['event_ids']
flavors = np.array(fin['flavors'])
energies = np.array(fin['energies'])
ccncs = np.array(fin['interaction_type'])
xx = np.array(fin['xx'])
yy = np.array(fin['yy'])
zz = np.array(fin['zz'])
zeniths = np.array(fin['zeniths'])
azimuths = np.array(fin['azimuths'])
inelasticity = np.array(fin['inelasticity'])
n_events = len(event_ids)

nuflavorint = (np.absolute(flavors) - 10) / 2
nu_nubar = (-1 * np.sign(flavors) + 1) / 2
pnu = np.log10(energies)
currentint = np.ones(n_events)
posnu_r = np.sqrt((xx) ** 2 + (yy) ** 2 + (-1.0 * zz - AraStationDep) ** 2)
posnu_phi = np.ones(n_events)
posnu_theta = np.ones(n_events)
print("AraStationDep = " + str(AraStationDep))
for i in range(n_events):
    if (ccncs[i] == "cc"):
        currentint[i] = 1
    else:
        currentint[i] = 0
    if (yy[i] >= 0):
        posnu_phi[i] = np.arccos(xx[i] / np.sqrt((xx[i]) ** 2 + (yy[i]) ** 2))
    else:
        posnu_phi[i] = -1.0 * np.arccos(xx[i] / np.sqrt(xx[i] ** 2 + yy[i] ** 2)) + 2.0 * np.pi
    if (-1.0 * zz[i] >= AraStationDep):
        posnu_theta[i] = -1.0 * np.arccos(np.sqrt(xx[i] ** 2 + yy[i] ** 2) / posnu_r[i])
    else:
        posnu_theta[i] = np.arccos(np.sqrt(xx[i] ** 2 + yy[i] ** 2) / posnu_r[i])

with open(args.AraSiminput, 'w') as fout:
    fout.write("//VERSION=0.1\n//EVENT_NUM=" + str(n_events) + "\n//evid nuflavorint nu_nubar pnu currentint posnu_r posnu_theta posnu_phi nnu_theta nnu_phi elast_y\n")
#    fout.write("//VERSION=0.1\n//EVENT_NUM=" + str(n_events) + "\n//evid nuflavorint nu_nubar pnu currentint posnu_r posnu_theta posnu_phi nnu_theta nnu_phi\n")
    for i in range(n_events):
#        fout.write("{:08d} {:>+5d}  {:.5e}  {:s}  {:>10.3f}  {:>10.3f}  {:>10.3f}  {:>10.3f}  {:>10.3f}  {:>10.3f}\n".format(event_ids[i], flavors[i], energies[i], ccncs[i], xx[i], yy[i], zz[i], zeniths[i], azimuths[i], inelasticity[i]))
        fout.write("{:08d} {:01d} {:01d} {:.3f} {:01d} {:.4f} {:.4f} {:.4f} {:.4f} {:.4f} {:.4f}\n".format(event_ids[i], int(nuflavorint[i]), int(nu_nubar[i]), pnu[i], int(currentint[i]), posnu_r[i], posnu_theta[i], posnu_phi[i], np.pi - zeniths[i], np.pi + azimuths[i], inelasticity[i]))
#        fout.write("{:08d} {:01d} {:01d} {:.3f} {:01d} {:.4f} {:.4f} {:.4f} {:.4f} {:.4f}\n".format(event_ids[i], int(nuflavorint[i]), int(nu_nubar[i]), pnu[i], int(currentint[i]), posnu_r[i], posnu_theta[i], posnu_phi[i], np.pi - zeniths[i], np.pi + azimuths[i]))
    fout.close()



from __future__ import absolute_import, division, print_function
import numpy as np
from NuRadioMC.utilities import units
from radiotools import helper as hp
from radiotools import plthelpers as php
import argparse
import h5py

parser = argparse.ArgumentParser(description='Run NuRadioMC simulation')
parser.add_argument('ASCIIoutput', type=str, help='path to ASCII output file')
parser.add_argument('hdf5input', type=str, nargs = '+', help='path to NuRadioMC hdf5 input event list')
args = parser.parse_args()

fin = h5py.File(args.hdf5input[0], 'r')
event_ids = np.array(fin['event_ids'])
flavors = np.array(fin['flavors'])
energies = np.array(fin['energies'])
ccncs = np.array(fin['interaction_type'])
xx = np.array(fin['xx'])
yy = np.array(fin['yy'])
zz = np.array(fin['zz'])
zeniths = np.array(fin['zeniths'])
azimuths = np.array(fin['azimuths'])
inelasticity = np.array(fin['inelasticity'])
weight = np.array(fin['weights'])
st = np.array(fin['ray_tracing_solution_type'])
launch_vectors = np.array(fin['launch_vectors'])
receive_vectors = np.array(fin['receive_vectors'])

for i in range(len(args.hdf5input) - 2):
    fin = h5py.File(args.hdf5input[i + 1], 'r')
    event_ids = np.append(event_ids, 10000 * (i + 1) + np.array(fin['event_ids']))
    flavors = np.append(flavors, np.array(fin['flavors']))
    energies = np.append(energies, np.array(fin['energies']))
    ccncs = np.append(ccncs, np.array(fin['ccncs']))
    xx = np.append(xx, np.array(fin['xx']))
    yy = np.append(yy, np.array(fin['yy']))
    zz = np.append(zz, np.array(fin['zz']))
    zeniths = np.append(zeniths, np.array(fin['zeniths']))
    azimuths = np.append(azimuths, np.array(fin['azimuths']))
    inelasticity = np.append(inelasticity, np.array(fin['inelasticity']))
    weight = np.append(weight, np.array(fin['weights']))
    st = np.append(st, np.array(fin['ray_tracing_solution_type']), axis = 0)
    launch_vectors = np.append(launch_vectors, np.array(fin['launch_vectors']), axis = 0)
    receive_vectors = np.append(receive_vectors, np.array(fin['receive_vectors']), axis = 0)

shower_axis = -1.0 * hp.spherical_to_cartesian(zeniths, azimuths)
viewing_angles_d = np.array([hp.get_angle(x, y) for x, y in zip(shower_axis, launch_vectors[:, 5, 0])])
viewing_angles_r = np.array([hp.get_angle(x, y) for x, y in zip(shower_axis, launch_vectors[:, 5, 1])])
launch_angles_d, launch_azimuths_d = hp.cartesian_to_spherical_vectorized(launch_vectors[:, 5, 0, 0].flatten(), launch_vectors[:, 5, 0, 1].flatten(), launch_vectors[:, 5, 0, 2].flatten())
launch_angles_r, launch_azimuths_r = hp.cartesian_to_spherical_vectorized(launch_vectors[:, 5, 1, 0].flatten(), launch_vectors[:, 5, 1, 1].flatten(), launch_vectors[:, 5, 1, 2].flatten())
rec_angles_d, rec_azimuths_d = hp.cartesian_to_spherical_vectorized(receive_vectors[:, 5, 0, 0].flatten(), receive_vectors[:, 5, 0, 1].flatten(), receive_vectors[:, 5, 0, 2].flatten())
rec_angles_r, rec_azimuths_r = hp.cartesian_to_spherical_vectorized(receive_vectors[:, 5, 1, 0].flatten(), receive_vectors[:, 5, 1, 1].flatten(), receive_vectors[:, 5, 1, 2].flatten())
n_events = len(event_ids)

with open(args.ASCIIoutput, 'w') as fout:
    fout.write(fin.attrs['header'])
    fout.write("event_ids flavors energies ccnc xx yy zz zeniths azimuths inelasticity weight\n")
    fout.write("viewing_angles_0 viewing_angles_1 launch_angles_0 launch_angles_1 rec_angles_0 rec_angles_1 solution_type_0 solution_type_1\n")
    for i in range(n_events):
        print(str(event_ids[i]) + ", ", end = '')
        fout.write("{:08d}  {:>+5d}  {:.5e}  {:s}  {:>10.3f}  {:>10.3f}  {:>10.3f}  {:>10.3f}  {:>10.3f}  {:>10.3f}  {:>10.3f}\n{:>10.3f}  {:>10.3f}  {:>10.3f}  {:>10.3f}  {:>10.3f}  {:>10.3f}  {:02f}  {:02f}\n".format(int(event_ids[i]), int(flavors[i]), energies[i], ccncs[i], xx[i], yy[i], zz[i], zeniths[i] / units.deg, azimuths[i] / units.deg, inelasticity[i], weight[i], viewing_angles_d[i] / units.deg, viewing_angles_r[i] / units.deg, launch_angles_d[i] / units.deg, launch_angles_r[i] / units.deg, 180.0 - rec_angles_d[i] / units.deg, 180.0 - rec_angles_r[i] / units.deg, st[i][5][0], st[i][5][1]))
    fout.close()



import numpy as np
from NuRadioMC.utilities import units

model_to_int = {"SP1" : 1}

#     def get_temperature(self, z):
#         return (-51.5 + z * (-4.5319e-3 + 5.822e-6 * z))

def get_temperature(z):
    """
    returns the temperature in Celsius as a function of depth
    """
    # from https://icecube.wisc.edu/~araproject/radio/#icetabsorption

    z2 = np.abs(z / units.m)
    return 1.83415e-09 * z2**3 + (-1.59061e-08 * z2**2) + 0.00267687 * z2 + (-51.0696)

def get_attenuation_length(z, frequency, model):
    if(model == "SP1"):
        t = get_temperature(z)
        f0 = 0.0001
        f2 = 3.16
        w0 = np.log(f0)
        w1 = 0.0
        w2 = np.log(f2)
        w = np.log(frequency / units.GHz)
        b0 = -6.74890 + t * (0.026709 - t * 0.000884)
        b1 = -6.22121 - t * (0.070927 + t * 0.001773)
        b2 = -4.09468 - t * (0.002213 + t * 0.000332)
        if(not hasattr(frequency, '__len__')):
            if (frequency < 1. * units.GHz):
                a = (b1 * w0 - b0 * w1) / (w0 - w1)
                bb = (b1 - b0) / (w1 - w0)
            else:
                a = (b2 * w1 - b1 * w2) / (w1 - w2)
                bb = (b2 - b1) / (w2 - w1)
        else:
            a = np.ones_like(frequency) * (b2 * w1 - b1 * w2) / (w1 - w2)
            bb = np.ones_like(frequency) * (b2 - b1) / (w2 - w1)
            a[frequency < 1. * units.GHz] = (b1 * w0 - b0 * w1) / (w0 - w1)
            bb[frequency < 1. * units.GHz] = (b1 - b0) / (w1 - w0)
    
        return 1. / np.exp(a + bb * w)
    else:
        raise NotImplementedError("attenuation model {} is not implemented.".format(model))


import NuRadioMC.SignalGen.askaryan as ask
from NuRadioMC.utilities import units, fft
import numpy as np
import matplotlib.pyplot as plt
from radiotools import plthelpers as php
import logging
logging.basicConfig(level=logging.INFO)

interp_factor = 20
# HAD for different viewing angles
E = 1 * units.EeV
shower_type = "HAD"
n_index = 1.78
theta_C = np.arccos(1. / n_index)
N = 512*2
dt = 0.1 * units.ns
R = 1 * units.km
tt = np.arange(0, dt * N, dt)

thetas = theta_C + np.array([0, 1, 2, 3, 4, 5]) * units.deg
fig, ax = plt.subplots(2, 3, sharex=True)
ax = ax.flatten()
for iTheta, theta in enumerate(thetas):
    trace = ask.get_time_trace(E, theta, N, dt, shower_type, n_index, R, "ARZ2019", interp_factor=interp_factor)
    ax[iTheta].plot(tt, trace[1], '-C0'.format(iTheta), label="$\Delta \Theta$ = {:.1f}".format((theta-theta_C)/units.deg))
    trace = ask.get_time_trace(E, theta, N, dt, shower_type, n_index, R, "Alvarez2000")
    trace = np.roll(trace, int(-1 * units.ns/dt))
    ax[iTheta].plot(tt, trace, '--C1'.format(iTheta), label="$\Delta \Theta$ = {:.1f}".format((theta-theta_C)/units.deg))
    ax[iTheta].set_title("$\Delta \Theta$ = {:.1f}".format((theta-theta_C)/units.deg))
    ax[iTheta].set_xlim(45, 60)
# ax.legend()
fig.tight_layout()
fig.suptitle("HAD, Esh = {:.1f}EeV".format(E/units.EeV))
fig.subplots_adjust(top=0.9)
fig.savefig("plots/04_1EeV_HAD.png")

shower_type = "EM"
fig, ax = plt.subplots(2, 3, sharex=True)
ax = ax.flatten()
for iTheta, theta in enumerate(thetas):
    trace = ask.get_time_trace(E, theta, N, dt, shower_type, n_index, R, "ARZ2019", interp_factor=interp_factor)
    ax[iTheta].plot(tt, trace[1], '-C0'.format(iTheta), label="$\Delta \Theta$ = {:.1f}".format((theta-theta_C)/units.deg))
    trace = ask.get_time_trace(E, theta, N, dt, shower_type, n_index, R, "Alvarez2000")
    trace = np.roll(trace, int(-1 * units.ns/dt))
    ax[iTheta].plot(tt, trace, '--C1'.format(iTheta), label="$\Delta \Theta$ = {:.1f}".format((theta-theta_C)/units.deg))
    ax[iTheta].set_title("$\Delta \Theta$ = {:.1f}".format((theta-theta_C)/units.deg))
    ax[iTheta].set_xlim(45, 70)
# ax.legend()
fig.tight_layout()
fig.suptitle("EM, Esh = {:.1f}EeV".format(E/units.EeV))
fig.subplots_adjust(top=0.9)
fig.savefig("plots/04_1EeV_EM.png")


shower_type = "EM"
E = 0.01 * units.EeV
fig, ax = plt.subplots(2, 3, sharex=True)
ax = ax.flatten()
for iTheta, theta in enumerate(thetas):
    trace = ask.get_time_trace(E, theta, N, dt, shower_type, n_index, R, "ARZ2019", interp_factor=interp_factor)
    ax[iTheta].plot(tt, trace[1], '-C0'.format(iTheta), label="$\Delta \Theta$ = {:.1f}".format((theta-theta_C)/units.deg))
    trace = ask.get_time_trace(E, theta, N, dt, shower_type, n_index, R, "Alvarez2000")
    trace = np.roll(trace, int(-1 * units.ns/dt))
    ax[iTheta].plot(tt, trace, '--C1'.format(iTheta), label="$\Delta \Theta$ = {:.1f}".format((theta-theta_C)/units.deg))
    ax[iTheta].set_title("$\Delta \Theta$ = {:.1f}".format((theta-theta_C)/units.deg))
    ax[iTheta].set_xlim(45, 70)
# ax.legend()
fig.tight_layout()
fig.suptitle("EM, Esh = {:.1f}PeV".format(E/units.PeV))
fig.subplots_adjust(top=0.9)
fig.savefig("plots/04_10PeV_EM.png")
plt.show()


"""
Basic checks for data dictionaries.
"""

import sys
from NuRadioMC.utilities.hdf5_manipulator import msg


def get_size(data):

    """Check if #entries is the same for all keys and return it

    Keyword arguments:
    data -- data dictionary
    """

    sizes = [d.shape[0] for d in data.itervalues()]  # shape[0] = #entries

    if max(sizes) != min(sizes):
        msg.error("Each dataset within a file must have the "
                  "same number of entries!")
        sys.exit(1)

    return sizes[0]


def same_sizes(data1, data2):

    """Check if files have the same #entries per dataset.

    Keyword arguments:
    data1 -- first file
    data2 -- second file
    """

    if get_size(data1) != get_size(data2):
        msg.error("Files must have the same number of entries to be combined.")
        sys.exit(1)


def check_keys(data1, data2):

    """Check it both files have the same datasets.

    Keyword arguments:
    data1 -- current data dictionary
    data2 -- data dictionary to be added
    """

    if data1.keys() != data2.keys():
        msg.error("Files have different datasets.")
        sys.exit(1)


def check_shapes(data1, data2):

    """Check if shapes of datasets are the same.

    Keyword arguments:
    data1 -- current data dictionary
    data2 -- data dictionary to be added
    """

    for key in data1.keys():
        if data1[key].shape[1:] != data2[key].shape[1:]:
            msg.error("Different shapes for dataset: %s. " % key)
            sys.exit(1)


def key_exists(key, data, filename):

    """Check if given dataset is included in the file.

    Keyword arguments:
    key -- key to look for
    data -- data dictionary to check
    """

    if key not in data.keys():
        msg.error("'%(key)s' key is missing in %(file)s."
                  % {"key": key, "file": filename})
        sys.exit(1)


def different_keys(data1, data2, skip):

    """Check if given files have different (except skip) datasets.

    Keyword arguments:
    data1 -- data dictionary
    data2 -- data dictionary
    skip -- common key
    """

    for key in data1.keys():
        if key == skip:
            continue
        if key in data2.keys():
            msg.error("Duplicated dataset: %s in input files." % key)
            sys.exit(1)


def check_duplicates(keys1, keys2):

    """Check if given files have different (except skip) datasets.

    Keyword arguments:
    keys1 -- the list of keys to be copied from file1
    keys2 -- the list of keys to be copied from file2
    """

    for key in keys1:
        if key in keys2:
            msg.error("Duplicated dataset: %s in input files." % key)
            sys.exit(1)
